#!/usr/bin/python
#
# BIND Configuration GUI Import Facility
#
# Jason Vas Dias, Red Hat Inc., Oct 2004

import sys
import os
import time
import copy
import gtk
import gtk.glade
import gnome.ui
import gnome
import gobject
import pango

import DNS
import RRGUI
from Hosts import *

class Import:
    
    def importHost(s):
        gobject.source_remove(s.sid)
        if len(s.hostEntries):
            ip, name, aliases = s.hostEntries.pop(0)
            debug( 'IMPORT:', ip, name, aliases)
            s.bind.add_host(ip,name,aliases,s.view)
            s.done += s.step
            s.pb.set_fraction(s.done)
        if len(s.hostEntries):
            s.sid=gobject.timeout_add(20, s.importHost)
        else:
            s.dialog.destroy()

    def doit(s):
        gobject.source_remove(s.sid)
        op=Filter.AND
        if s.filterOpOR.get_active():
            op=Filter.OR
        s.hostEntries=[]
        a=s.hosts.filter(s.hostEntries, op=op)
        if len(s.hostEntries) > 0 :
            s.step = 1.0/float(len(s.hostEntries))
            s.done=0.0
            s.sid=gobject.timeout_add(1, s.importHost)
        else:
            s.bind.gui.modalError(s.bind.gui.gui,_('Filter Error'),_('No hosts are selected by the filter.'))
        return False

    def answer( s, response, arg=None):
        s.response(s.dialog, int( gtk.RESPONSE_OK ), response)
        return False

    def response(s, d, r, answer=None):
        debug( 'RESPONSE:',r)
        if r == gtk.RESPONSE_OK :            
            k=[]
            if  hasattr(s,'hosts'):
                k = s.hosts.keys()
            if( len(k) <= 0):
                s.bind.gui.modalError(s.bind.gui.gui,_('Import File Error'),_('First choose a valid hosts file.'))
                return False
            s.save_filters()
            if (len(s.hosts.filters) <= 0):
                if(answer == None):
                    ask = s.bind.gui.question(
                          s.bind.gui.gui,
                          _('File Import'),
                          _('No filters are defined - ALL hosts in the file will be imported.\nDo you want to import ALL hosts in the file?'), modal=0)
                    ask.question(s.answer)
                    return False
                elif (answer == gtk.RESPONSE_CANCEL):
                    return False
            s.sid = gobject.timeout_add(100,s.doit)
            return False
        s.dialog.destroy()
        return False
        
    def file_changed(s, w, e):
        debug( 'FILE CHANGED')
        file = s.fileEntry.get_text()
        if( file != s.file ):
            s.fileChooser.set_filename(file)
            s.file_chosen(w,gtk.RESPONSE_OK)        
        return False
    
    def file_dir(s):
        file=s.fileEntry.get_text()
        i=file.rfind('/')
        if (i > -1):
            return ( file[-(len(file)-(i+1)):], file[0:i] )
        return ('./',file)    

    def file_chosen(s, w, response_enum):
        file=s.fileChooser.get_filename()
        response=int(response_enum)
        debug( 'CHOOSE FILE', file, response)
        if( response != gtk.RESPONSE_CANCEL ):
            if (file==None) or os.access(file, os.F_OK | os.R_OK) != 1:
                s.bind.gui.modalError(s.bind.gui.gui,_('Import File Error'),_('You must choose a file that exists and that you have read permission for.'))
                s.fileChooser.set_filename('/etc/hosts')
                s.fileEntry.set_text('/etc/hosts')
                return False
            filters=[]
            if hasattr(s,'hosts'):
                filters=s.hosts.filters            
            s.hosts = Hosts(file)
            s.hosts.filters=filters
            k = s.hosts.hosts.keys()
            if (k==None) or (len(k) <= 0):
                s.bind.gui.modalError(s.bind.gui.gui,_('Import File Error'),
                                      _('File "%s" is an invalid hosts file.') % file
                                     )
                s.fileChooser.set_filename('/etc/hosts')
                s.fileEntry.set_text('/etc/hosts')
                return False
        s.file = file
        s.fileEntry.set_text(file)
        s.fileChooser.hide()
        return False                        

    def hide_chooser(s,w,e):
        s.fileChooser.hide()
        
    def browse_button(s, b):
        s.fileChooser.show()

    def selected(s,tree):
        selection = tree.get_selection()
        if( selection != None ):
            model, selected = selection.get_selected()
            if (model != None) and  (selected != None):
                return (selected,) + model.get(selected, 0, 1)
        return (None,None,None)

    def save_filter(s, model, path, iter, user_data=None):
        ele = model.get_value(iter, 1)
        debug( 'FILTER:',str(ele))
        s.hosts.filters.append(ele)
        
    def save_filters(s):
        s.hosts=Hosts(s.file)
        s.hosts.filters=[]
        s.filterList.foreach(s.save_filter)
        
    def remove_filter_element(s,b,iter):
        s.filterList.remove(iter)
        if s.filterList.iter_n_children(None) > 0:
            s.filterTree.set_cursor((0,))
        s.filter_element_change(s.filterTree,1)
            
    def save_filter_element(s,b,iter):
        f=s.filterList.get_value(iter, 1).__class__( s.editWidget.value() )
        if s.notButton.get_active() :
            f.notted = True
        s.filterList.set(iter, 0, str(f), 1, f)

    def add_filter_element(s,b):
        if not hasattr(s,'hosts'):
            s.hosts=Hosts(s.fileEntry.get_text())
        f = s.hosts.add_filter( s.editWidget.value() )
        if s.notButton.get_active() :
            f.notted = True
        iter = s.filterList.append([ str(f), f ])
        s.filterTree.set_cursor(s.filterList.get_path(iter))
        s.filter_element_change( s.filterTree,1)

    def toggle_not(s,b,attr):
        if( b.get_active() ):
            b.set_label("!")
        else:
            b.set_label(" ")
        return False
            
    def edit_filter_element(s, attr, iter):
        editContainer = gtk.VBox()
        s.eHB=gtk.HBox()        
        s.okButton = gtk.Button(stock=gtk.STOCK_OK)
        debug( 'EDIT:',attr.__class__,iter)
        s.notButton = gtk.ToggleButton()
        if hasattr(attr,'notted') and attr.notted:
            s.notButton.set_active(True)
            s.notButton.set_label("!")
        else:
            s.notButton.set_active(False)
            s.notButton.set_label(" ")
        if issubclass(attr.__class__, Filter):
            debug( 'EDIT FILTER',attr.__class__,iter)
            s.editWidget = s.bind.gui.widget(attr.attr())(s, s, attr.attr(), None)
            s.removeButton = gtk.Button(stock=gtk.STOCK_REMOVE)
            s.removeButton.connect('clicked',s.remove_filter_element,iter)
            s.eHB.pack_start(s.removeButton)
            s.okButton.connect('clicked',s.save_filter_element,iter)
            s.notButton.connect("toggled",s.toggle_not,attr)
        else:
            debug( 'EDIT NEW FILTER:',attr.__class__,iter)
            new_attr=copy.deepcopy(attr)
            s.editWidget = s.bind.gui.widget(new_attr)(s, s, new_attr, None)
            s.okButton.connect('clicked',s.add_filter_element)
            s.notButton.connect("toggled",s.toggle_not,new_attr)
        s.eHB.pack_start(s.okButton)
        s.ewHB = gtk.HBox()
        s.ewHB.pack_start(s.notButton)
        s.ewHB.pack_start(s.editWidget.widget())
        editContainer.pack_start(s.ewHB)
        editContainer.pack_start(s.eHB)
        return editContainer
    
    def filter_element_change(s, tree, new=None):
        iter,as,a=s.selected(tree)
        if s.editContainer != None:
            s.editContainer.hide()
            s.editHB.remove(s.editContainer)
            s.editContainer.destroy()
        if( new == None):
            iter = None
        if( a != None):
            attr=a
            s.editContainer = s.edit_filter_element(attr,iter)
            s.editHB.pack_start(s.editContainer)
        else:
            s.editContainer = None
        s.editHB.show_all()

    def filter_element_activated(s,tree,path,column):
        s.filter_element_change(tree,path)
        return False
    
    def new_filter_element_activated(s,tree,path,column):
        s.filter_element_change(tree)
        return False
    
    def filter_element_selected(s, tree, event):
        s.filter_element_change(tree,1)
        return False

    def new_filter_element_selected(s, tree, event):
        s.filter_element_change(tree)
        return False
    
    def __init__(s,zoneTree,view=None):
        s.zoneTree = zoneTree
        s.bind = s.zoneTree.bind
        s.view = view
        s.file = '/etc/hosts'
        s.dialog = gtk.Dialog(title=_("Import Hosts File"),
                              parent=s.bind.gui.gui,
                              buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
                                       gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL
                                      )
                             )
        s.editDialog = s.dialog
        s.gui=s.dialog
        s.dialog.connect('response', s.response)
        s.fileFrame=gtk.Frame(label=_("Select Hosts File"))
        s.fileHB=gtk.HBox()
        s.fileEntry = gtk.Entry()
        s.fileEntry.set_text("/etc/hosts")
        s.fileEntry.connect('focus_out_event', s.file_changed)
        s.fileChooseButton = gtk.Button(label='Browse...',stock=gtk.STOCK_OPEN)
        s.fileChooseButton.connect('clicked', s.browse_button)
        s.fileHB.pack_start(s.fileEntry)
        s.fileHB.pack_start(s.fileChooseButton)
        s.fileFrame.add(s.fileHB)
        s.dialog.vbox.pack_start(s.fileFrame)
        #
        s.fileChooser = gtk.FileChooserDialog(title=_('Choose Hosts File to Import'),
                                              action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                              buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,
                                                       gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL
                                                      )
                                             )
        s.fileChooser.connect('response',s.file_chosen)
        s.fileChooser.connect('delete_event',s.hide_chooser)
        s.fileChooser.set_filename(s.file)
        s.filterFrame=gtk.Frame(label=_('Filter Host Entries'))
        s.filterHB=gtk.HBox()
        s.filterList = gtk.ListStore( gobject.TYPE_STRING, gobject.TYPE_PYOBJECT )
        s.newList = gtk.ListStore( gobject.TYPE_STRING, gobject.TYPE_PYOBJECT )
        s.filterTree = gtk.TreeView( model=s.filterList )
        s.newTree = gtk.TreeView( model=s.newList )
        s.filterTree.set_headers_visible(False)
        s.newTree.set_headers_visible(False)
        s.filterSW =   gtk.ScrolledWindow()
        s.filterSW.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
        s.newSW =   gtk.ScrolledWindow()
        s.newSW.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
        s.filterTree.connect('row_activated',s.filter_element_activated)
        s.newTree.connect('row_activated',s.new_filter_element_activated)
        s.filterTree.connect('button_release_event',s.filter_element_selected)
        s.newTree.connect('button_release_event',s.new_filter_element_selected) 
        s.filterCell = gtk.CellRendererText()
        s.newCell = gtk.CellRendererText()
        s.filterColm = gtk.TreeViewColumn( None, s.filterCell, text=0 )
        s.newColm = gtk.TreeViewColumn( None, s.newCell, text=0 )
        s.filterTree.append_column(s.filterColm)
        s.newTree.append_column(s.newColm)        
        s.filterListFrame = gtk.Frame(label=_('Filter List'))
        s.filterSW.add_with_viewport(s.filterTree)
        s.filterVB=gtk.VBox()
        s.filterVB.pack_start(s.filterSW)
        s.filterListFrame.add(s.filterVB)
        s.editFrame = gtk.Frame(label=_('Edit List Element'))        
        s.editHB = gtk.HBox()
        s.editContainer= gtk.VBox()
        s.editWidget = gtk.Label('')        
        s.editContainer.pack_start(s.editWidget)
        s.editHB.pack_start(s.editContainer)
        s.editFrame.add(s.editHB)
        s.newFrame = gtk.Frame(label=_('New List Element'))
        s.newSW.add_with_viewport(s.newTree)
        s.newFrame.add(s.newSW)
        s.filterHB.pack_start(s.filterListFrame)
        s.filterHB.pack_start(s.editFrame)
        s.filterHB.pack_start(s.newFrame)
        s.newList.append([_('IPV4 Address Filter'), DNS.IPV4.Address('0.0.0.0/32',True,True)])
        s.newList.append([_('IPV6 Address Filter'), DNS.IPV6.Address('::/128',DNS.IPV6.HEX_FORMAT,True)])
        s.newList.append([_('DNS Name Filter'), '' ])
        s.filterVB=gtk.VBox()
        s.filterVB.pack_start(s.filterHB)
        s.filterOpHB=gtk.HBox()
        s.filterOpAND=gtk.RadioButton(label=_('Match ALL Filters'))
        s.filterOpOR=gtk.RadioButton(group=s.filterOpAND,label=_('Match ANY Filter'))
        s.filterOpAND.set_active(True)
        s.filterOpHB.pack_start(s.filterOpAND)
        s.filterOpHB.pack_start(s.filterOpOR)
        s.filterVB.pack_start(s.filterOpHB)
        s.filterFrame.add(s.filterVB)
        s.dialog.vbox.pack_start(s.filterFrame)
        s.pb=gtk.ProgressBar()
        s.pb.set_orientation(gtk.PROGRESS_LEFT_TO_RIGHT)
        s.pb.set_fraction(0.0)
        s.dialog.vbox.pack_start(s.pb)
        s.dialog.show_all()

        
