#!/usr/bin/python
#
# BIND named.conf Options
#
# Jason Vas Dias, Sept. 2004
import Lookup
import DNS

O = { 
    'version' :
    { 'type' : str,
      'name': _('Version String'),
      'desc': _('The version the server should report via a query of name version bind in class CHAOS.') +_(' The default is the real version number of this server.') ,
    },
    'directory' :
    { 'type': DNS.Path,
      'name': _('Zone File Directory'),
      'isDirectory':True,
      'isAbsolute':True,
      'default':'/var/named',
      'desc': _('The working directory of the server.') +_(' Any non-absolute pathnames in the configuration file will be taken as relative to this directory.') +_(' The default location for most server output files (e.g. named.run) is this directory.') +_(' If a directory is not specified, the working directory defaults to ".", the directory from which the server was started.') +_(' The directory specified should be an absolute path (should begin with "/").') 
    },
    'file':
    { 'type': DNS.Path,
      'name': _('Zone File Path'),
      'desc': _('The path of the Zone database file.') +_(' If the specified path is not absolute (does not begin with "/"), the path is implicitly relative to the path specified by the "Directory" option.') ,
      'zone': True,
      'zone-only': True,
      'zone-types': ['slave','master','stub','hint','delegation-only']
    },
    'type':
    { 'type': DNS.Radio,
      'name': _('Zone Type'),
      'values': ['master', 'slave', 'forward', 'hint', 'stub', 'delegation-only'],
      'zone': True,
      'zone-only': True,
      'read-only': True,
      'zone-types': ['master','slave','forward','stub','hint','delegation-only'],
      'desc': _('<b>master</b>: The server has a master copy of the data for the zone and will be able to provide authoritative answers for it.') +'\n'+_('<b>slave</b>: A slave zone is a replica of a master zone.') +_(' A slave zone MUST specify a "masters" list.') +_(' The masters list specifies one or more IP addresses of master servers that the slave contacts to update its copy of the zone.') +_(' By default, transfers are made from port 53 on the servers; this can be changed for all servers by specifying a port number before the list of IP addresses, or on a per-server basis after the IP address.') +_(' Authentication to the master can also be done with per-server TSIG keys.') +_(' If a file is specified, then the replica will be written to this file whenever the zone is changed, and reloaded from this file on a server restart.') +_(' Use of a file is recommended, since it often speeds server start-up and eliminates a needless waste of bandwidth, but is not required.') +_(' Note that for large numbers (in the tens or hundreds of thousands) of zones per server, it is best to use a two level naming scheme for zone file names.') +_(' For example, a slave server for the zone example.com might place the zone contents into a file called ex/example.com where ex/ is just the first two letters of the zone name.') +_(' (Most operating systems behave very slowly if you put 100K files into a single directory') +_(').\n<b>stub</b>A stub zone is similar to a slave zone, except that it replicates only the NS records of a master zone instead of the entire zone.') +_(' Stub zones are not a standard part of the DNS; they are a feature specific to the BIND implementation.') +_(' Stub zones can be used to eliminate the need for glue NS record in a parent zone at the expense of maintaining a stub zone entry and a set of name server addresses in named.conf.') +_(' This usage is not recommended for new configurations, and BIND 9 supports it only in a limited way.') +_(' In BIND 4/8, zone transfers of a parent zone included the NS records from stub children of that zone.') +_(' This meant that, in some cases, users could get away with configuring child stubs only in the master server for the parent zone.') +_(' BIND 9 never mixes together zone data from different zones in this way.') +_(' Therefore, if a BIND 9 master serving a parent zone has child stub zones configured, all the slave servers for the parent zone also need to have the same child stub zones configured.') +_(' Stub zones can also be used as a way of forcing the resolution of a given domain to use a particular set of authoritative servers.') +_(' For example, the caching name servers on a private network using RFC1981 addressing may be configured with stub zones for 10.in-addr.arpa to use a set of internal name servers as the authoritative servers for that domain.') +_('\n<b>forward</b>: A "forward zone" is a way to configure forwarding on a per-domain basis.') +_(' A zone statement of type forward can contain a forward and/or forwarders statement, which will apply to queries within the domain given by the zone name.') +_(' If no forwarders statement is present or an empty list for forwarders is given, then no forwarding will be done for the domain, canceling the effects of any forwarders in the options statement.') +_(' Thus if you want to use this type of zone to change the behavior of the global forward option (that is, "forward first to", then "forward only", or vice versa, but want to use the same servers as set globally) you need to respecify the global forwarders.') +_('\n<b>hint</b>: The initial set of root nameservers is specified using a "hint zone".') +_(' When the server starts up, it uses the root hints to find a root nameserver and get the most recent list of root nameservers.') +_(' If no hint zone is specified for class IN, the server uses a compiled-in default set of root servers hints.') +_(' Classes other than IN have no built-in defaults hints.') +_('\n<b>delegation-only</b>: This is used to enforce the delegation only status of infrastructure zones (e.g. COM, NET, ORG).') +_(' Any answer that is received without a explicit or implicit delegation in the authority section will be treated as NXDOMAIN.') +_(' This does not apply to the zone apex.') +_(' This SHOULD NOT be applied to leaf zones.') +_(' delegation-only has no effect on answers received from forwarders.') ,
    },
    'named-xfer' :
    { 'name': _('named-xfer Program Path'), 
      'deprecated': True,    
      'type': DNS.Path,
      'desc': _('This option is obsolete.') +_(' It was used in BIND 8 to specify the pathname to the named-xfer program.') +_(' In BIND 9, no separate named-xfer program is needed; its functionality is built into the name server.') ,
    },
    'tkey-domain' :
    { 'type': DNS.Name ,
      'name': _('TKEY Domain Name'),
      'desc': _('The domain name appended to the names of all shared keys generated with TKEY.') +_(' When a client requests a TKEY exchange, it may or may not specify the desired name for the key.') +_(' If present, the name of the shared key will be "client specified part" + "tkey-domain".') +_(' Otherwise, the name of the shared key will be "random hex digits" + "tkey-domain".') +_(' In most cases, the domainname should be the server\'s domain name.') ,
    },
    'tkey-dhkey' :
    {  'name': _('TKEY Diffie-Hellman Key'),
       'desc': _('The Diffie-Hellman key used by the server to generate shared keys with clients using the Diffie-Hellman mode of TKEY.') +_(' The server must be able to load the public and private keys from files in the working directory.') +_(' In most cases, the keyname should be the server\'s host name.') ,
       'attrs':
       [
        { 'key_name':
          { 'name': _('Key Name'),
            'type': DNS.Name ,
            'desc': _('Name of key.') ,
            'required':1
          }
        },
        { 'key_tag':
          { 'name': _('Key Tag'),
            'type' :  int ,
            'desc' :  _('Integer tag of key.'),
            'required':1
          }
        },
      ],
    },
    'pubkey':
    { 'name': _('Public Key'),
      'type': str ,
      'deprecated':True,
      'zone': True,
      'zone-only': True,
      'zone-types': ['master','slave','forward','stub','hint','delegation-only'],
      'desc': _('In BIND 8, this option was intended for specifying a public zone key for verification of signatures in DNSSEC signed zones when they are loaded from disk.') +_(' BIND 9 does not verify signatures on loading and ignores the option.') ,
    },
    'dump-file' :
    { 'type': DNS.Path,
      'name': _('Dump File Path'),
      'default':'data/cache_dump.txt',
      'desc': _('The pathname of the file the server dumps the database to when instructed to do so with rndc dumpdb.') +_(' If not specified, the default is named_dump.db .') ,
    },
    'memstatistics-file':
    { 'unimplemented': True,
      'name': _('Memory Statistics File Path'),
      'type': DNS.Path,
      'desc': _('The pathname of the file the server writes memory usage statistics to on exit.') +_(' If not specified, the default is named.memstats.') +_('\n<b>Note:</b>Not yet implemented in BIND 9.')
    },
    'pid-file':
    { 'name': _('Process Identifier File Path'),
      'type': DNS.Path,
      'default':'/var/run/named/named.pid',
      'desc': _('The pathname of the file the server writes its process ID in.') +_(' If not specified, the default is /var/run/named.pid.') +_(' The pid-file is used by programs that want to send signals to the running nameserver.')
    },        
    'statistics-file' :
    { 'name': _('Statistics File Path'),
      'type': DNS.Path,
      'default':'data/named_stats.txt',
      'desc': _('The pathname of the file the server appends statistics to when instructed to do so using rndc stats.') +_(' If not specified, the default is named.stats in the server\'s current directory.')
    },
    'zone-statistics' :
    { 'name': _('Zone Statistics'),
      'type': DNS.Boolean,
      'values': [ _('Disable'), _('Enable') ],
      'default': 0,
      'desc': _('If enabled in Global Options, the server will, by default, collect statistical data on all zones in the server.') +_(' If enabled in Zone Options, the server will collect statistics for that zone.') +_(' These statistics may be accessed using rndc stats, which will dump them to the file listed in the statistics-file.') ,
      'zone': True,
      'zone-types': ['master','slave','forward','stub','hint','delegation-only']
    },
    'auth-nxdomain' :
    { 'name': _('NXDOMAIN Responses'),
      'type': DNS.Boolean,
      'values': [ _('Not Authoritative'), _('Authoritative') ],
      'default': 0,
      'desc' : _('If yes, then the AA bit is always set on NXDOMAIN responses, even if the server is not actually authoritative. The default is no; this is a change from BIND 8. If you are using very old DNS software, you may need to set it to yes.')
    },      
    'deallocate-on-exit' :
    { 'name': _('Memory Leakage Detection on Exit'),
      'type': DNS.Boolean,
      'values': [ _('Disable'), _('Enable') ],
      'default': 0,
      'deprecated': True,
      'desc' : _('This option was used in BIND 8 to enable checking for memory leaks on exit. BIND 9 ignores the option and always performs the checks.')
    },
    'dialup' :
    { 'name': _('Dialup Zone Transfer'),
      'type': DNS.Boolean,
      'values': [ _('Disable'), _('Enable') ],
      'default': 0,
      'desc': _('The server treats all Dialup zones (or ALL zones if Dialup is in Global Options) as if they are doing zone transfers across a dial on demand dialup link, which can be brought up by traffic originating from this server.') +_(' This has different effects according to zone type and concentrates the zone maintenance so that it all happens in a short interval, once every <b>Heartbeat interval</b> and hopefully during the one call.') +_(' It also suppresses some of the normal zone maintenance traffic.') +_(' The default is no.') +_('\nThe dialup option may also be specified in the view and zone statements, in which case it overrides the global dialup option.') +_('\nIf the zone is a master zone, then the server will send out a NOTIFY request to all the slaves.') +_(' This will trigger the zone serial number check in the slave (providing it supports NOTIFY) allowing the slave to verify the zone while the connection is active.') +_('\nIf the zone is a slave or stub zone, then the server will suppress the regular "zone up to date" (refresh) queries and only perform them when the <b>Heartbeat Interval</b> expires in addition to sending NOTIFY requests.') +_('\nFiner control can be achieved by using <b>Notify</b> which only sends NOTIFY messages, <b>Notify Passive</b> which sends NOTIFY messages and suppresses the normal refresh queries, <b>Refresh</b> which suppresses normal refresh processing and send refresh queries when the <b>Heartbeat Interval</b> expires and <b>Passive</b> which just disables normal refresh processing.') ,
      'zone': True,
      'zone-types': ['master','slave','stub','hint','delegation-only'],
    },
    'delegation-only':
    { 'name': _('Delegation Only'),
      'type': DNS.Boolean,
      'values': [  _('Disable'),_('Enable') ],
      'default': 0,
      'desc': _('The flag only applies to hint and stub zones.') +_(' If set to yes then the zone will also be treated as if it is also a delegation-only type zone.') ,
      'zone':True,
      'zone-only':True,
      'zone-types': ['stub','hint']
    },
    'fake-iquery' :
    { 'name': _('Fake IQUERY'),
      'deprecated': True,
      'type': DNS.Boolean,
      'values': [ _('Disable'), _('Enable')],
      'default':0,
      'desc': _('In BIND 8, this option was used to enable simulating the obsolete DNS query type IQUERY.') +_(' BIND 9 never does IQUERY simulation.') ,
    },
    'fetch-glue' :
    { 'name': _('Fetch Glue Records'),
      'deprecated': True,
      'type': DNS.Boolean,
      'values': [ _('Disable'), _('Enable')],
      'default': 0,
      'desc': _('This option is obsolete.') +_(' In BIND 8, enabling <b>Fetch Glue Records</b> caused the server to attempt to fetch glue resource records it didn\'t have when constructing the additional data section of a response.') +_(' This is now considered a bad idea and BIND 9 never does it.') ,
    },
    'has-old-clients' :
    { 'name': _('Old Clients'),
      'deprecated': True,
      'type': DNS.Boolean,      
      'values': [ _('Doesn\'t Have'), _('Has') ],
      'default': 0,
      'desc': _('This option was incorrectly implemented in BIND 8, and is ignored by BIND 9.') +_(' To achieve the intended effect of has-old-clients yes, specify the two separate options <b>Authoritative NXDOMAIN Responses</b><tt> = Enable</tt> and <b>RFC2308 Type1</b><tt> = Disable</tt> instead.') ,
    },
    'host-statistics' :
    { 'name': _('Host Statistics'),
      'deprecated': True,
      'type': DNS.Boolean,
      'values': [ _('Disable'), _('Enable') ],
      'default': 0,
      'desc': _('In BIND 8, this enables keeping of statistics for every host that the nameserver interacts with.') +_(' Not implemented in BIND 9.')
    },
    'maintain-ixfr-base':
    { 'name': _('Maintain IXFR Base'),
      'deprecated': True,
      'type': DNS.Boolean,
      'values': [ _('Disable'), _('Enable') ],
      'default': 0,
      'desc': _('This option is obsolete.') +_(' It was used in BIND 8 to determine whether a transaction log was kept for Incremental Zone Transfer.') +_(' BIND 9 maintains a transaction log whenever possible.') +_(' If you need to disable outgoing incremental zone transfers, use <b>Provide IXFR</b><tt>=Disable</tt>.') ,
      'zone':True,
      'zone-types': ['master','slave','stub','delegation-only']
    },
    'ixfr-base':
    { 'name': _('Incremental Zone Transfer Log File Path'),
      'type': DNS.Path,
      'deprecated': True,
      'zone': True,
      'zone-only': True,
      'zone-types': ['master','slave','forward','stub','hint','delegation-only'],
      'desc': _('Was used in BIND 8 to specify the name of the transaction log (journal) file for dynamic update and IXFR.') +_(' BIND 9 ignores the option and constructs the name of the journal file by appending ".jnl" to the name of the zone file.') ,
    },
    'ixfr-tmp-file':
    { 'name': _('Incremental Zone Transfer Temporary File Path'),
      'type': DNS.Path,
      'deprecated': True,
      'zone': True,
      'zone-only': True,
      'zone-types': ['master','slave','forward','stub','hint','delegation-only'],
      'desc': _('Was an undocumented option in BIND 8.') +_(' Ignored in BIND 9.')
    },
    'minimal-responses' :
    { 'name': _('Minimal Responses'),
      'type': DNS.Boolean,
      'values': [ _('Disable'), _('Enable') ],
      'default': 0,
      'desc': _('If Enabled, then when generating responses the server will only add records to the authority and additional data sections when they are required (e.g. delegations, negative responses).') +_(' This may improve the performance of the server.') +_(' The default is Disabled.')
    },
    'multiple-cnames' :
    { 'name': _('Multiple CNAMES'),
      'deprecated': True,
      'type': DNS.Boolean,
      'values': [ _('Allow'), _('Disallow') ],
      'default': 0,
      'desc': _('This option was used in BIND 8 to allow a domain name to allow multiple CNAME records in violation of the DNS standards.') +_(' BIND 9.2+ always strictly enforces the CNAME rules both in master files and dynamic updates.') ,
    },      
    'notify':
    { 'name': _('Notify'),
      'type': DNS.Radio,
      'values': [ 'no', 'yes', 'explicit'],
      'default': 1,
      'desc': _('If Notify is <tt>yes</tt> (the default), DNS NOTIFY messages are sent when a zone the server is authoritative for changes.') +_(' The messages are sent to all the servers listed in the zone\'s NS records (except the master server identified in the SOA MNAME field), and to any servers listed in the <b>Also Notify</b> option.') +_('\nIf <tt>explicit</tt>, NOTIFYs are sent only to servers explicitly listed using <b>Also Notify</b>.') +_(' If <tt>none</tt>, no notifies are sent.') +_('\nThe <b>Notify</b> option may also be specified in the zone statement, in which case it overrides the <b>Options</b> <b>Notify</b> statement.') +_(' It would only be necessary to turn off this option if it caused slaves to crash.') ,
      'zone': True,
      'zone-types': ['master','stub','hint','delegation-only']
    },
    'recursion' :
    { 'name': _('Recursion'),
      'type': DNS.Boolean,
      'values': [_('Disallow'), _('Allow')],
      'default':1,
      'desc': _('If <b>Recursion</b>=<tt>Allow</tt>, and a DNS query requests recursion, then the server will attempt to do all the work required to answer the query.') +_(' If <b>Recursion</b> is not <tt>Allow</tt>ed and the server does not already know the answer, it will return a referral response.') +_(' The default is <tt>Allow</tt>.') +_(' Note that setting <b>Recursion</b>=<tt>Disallow</tt> does not prevent clients from getting data from the server\'s cache; it only prevents new data from being cached as an effect of client queries.') +_(' Caching may still occur as an effect of the server\'s internal operation, such as NOTIFY address lookups.') +_(' See also the documentation for the <b>Fetch Glue</b> option.')
    },
    'rfc2308-type1'  :
    { 'name': _('RFC2308 Type1'),
      'type': DNS.Boolean,
      'unimplemented': True,
      'values': [ _('Disable'), _('Enable') ],
      'default': 0,
      'desc': _('Setting this to <tt>Enable</tt> will cause the server to send NS records along with the SOA record for negative answers.') +_(' The default is <tt>Disable</tt>.') +_('\n<b>Note</b>: Not yet implemented in BIND 9.')
    },
    'use-id-pool' : 
    { 'name': _('Use ID Pool'),
      'type': DNS.Boolean,
      'deprecated': True,
      'values': [ _('Disable'), _('Enable') ],
      'default': 1,
      'desc': _('This option is obsolete.') +_(' BIND 9 always allocates query IDs from a pool.')
    },   
    'forward':
    { 'name': _('Forward'),
      'type': DNS.Radio,
      'values': [ 'first', 'only' ],
      'default': 0,
      'desc': _('This option is only meaningful if the <b>Forwarders</b> list is not empty.') +_(' A value of <tt>first</tt>, the default, causes the server to query the forwarders first, and only if they do not answer the question the server will itself then look for the answer.') +_(' If <tt>only</tt> is specified, the server will only query the forwarders and not look for the answer itself.') ,
      'zone':True,
      'zone-types': ['forward'],
    },
    'forwarders':
    { 'name': _('Forwarders'),
      'type': DNS.AddressList,
      'desc': _('Specifies the IP addresses to be used for forwarding.') +_(' The default is the empty list (no forwarding).') +_('\nForwarding can also be configured on a per-Zone basis, allowing for the global forwarding options to be overridden in a variety of ways.') +_(' You can set particular domains to use different forwarders, or have a different forward only/first behavior, or not forward at all.') ,
      'zone': True,
      'zone-types': ['forward'],
    },
    'masters':
    { 'name': _('Masters'),
      'type': DNS.AddressList,
      'desc': _('The masters list specifies one or more IP addresses of master servers that the slave contacts to update its copy of the zone.') ,
      'zone': True,
      'zone-only':True,
      'zone-types': ['slave'],
    },
    'check-names':
    { 'name': _('Check Names'),
      'attrs':
      [ { 'source':
          { 'name': _('Source'),
            'type': DNS.Radio,
            'values': [ 'master', 'slave', 'response' ],
            'required':1
          },
        },
        { 'disposition':
          { 'name': _('Disposition'),
            'type': DNS.Radio,
            'values': [ 'warn',  'fail', 'ignore' ],
            'required':1
          }
        }
      ],
      'desc': _('This option is fully implemented only in BIND 9.3.0 and is deprecated in previous versions.') +_('\nThe <b>Check Names</b> option enables checking names for RFC952 compliance.') +_(' To check names in type <tt>master</tt> zones, specify Source as <tt>master</tt>; to check names in IXFRs from a another name server, specify Source as <tt>slave</tt>; to check names in responses received from other name servers, specify Source as <tt>response</tt>.') +_(' If a name does not pass checking, the action taken by the server can be specified with Disposition: if Disposition is <tt>warn</tt>, a warning message will be written to the log; if <tt>fail</tt>, the server will write an error message to the log and exit; if <tt>ignore</tt>, the server will ignore the check failure and allow the name to be used.') ,
      'zone': True,
      'zone-types': ['master','stub','hint','delegation-only']
    }, 
    'allow-notify':
    { 'name': _('Allow Notify'),
      'type': DNS.ACL,
      'desc': _('Specifies which hosts are allowed to notify slaves of a zone change in addition to the zone masters.') +_(' <b>Allow Notify</b> may also be specified in Zone Options, in which case it overrides the global <b>Allow-Notify</b> option.') +_(' It is only meaningful for a slave zone.') +_(' If not specified, the default is to process notify messages only from a zone\'s master.') ,
      'zone': True,
      'zone-types': ['slave']
    },
    'allow-query':
    { 'name': _('Allow Query'),
      'type': DNS.ACL,
      'desc': _('Specifies which hosts are allowed to ask ordinary questions.') +_(' <b>Allow Query</b> may also be specified in Zone Options, in which case it overrides the global <b>Allow Query</b> option.') +_(' If not specified, the default is to allow queries from all hosts.') ,
      'zone': True,
      'zone-types': ['master','slave','forward','stub','hint','delegation-only']
    },
    'allow-transfer':
    { 'name': _('Allow Transfer'),
      'type': DNS.ACL,
      'desc': _('Specifies which hosts are allowed to receive zone transfers from the server.') +_(' <b>Allow Transfer</b> may also be specified in Zone Options, in which case it overrides the global <b>Allow Transfer</b> statement.') +_(' If not specified, the default is to allow transfers to all hosts.') ,
      'zone': True,
      'zone-types': ['master','stub','delegation-only']
    },
    'allow-update':
    { 'name': _('Allow Update'),
      'type': DNS.ACL,
      'desc': _('Allow Dynamic DNS Update from the hosts and/or networks specified by the access control list.') ,
      'zone': True,
      'zone-only': True,
      'zone-types': ['master','stub','delegation-only']
    },
    'update-policy':
    { 'name': _('Update Policy'),
      'type': DNS.UpdatePolicy,
      'desc': _('The update-policy clause is new in BIND 9 and allows more fine-grained control over what updates are allowed.') +_(' A set of rules is specified, where each rule either grants or denies permissions for one or more names to be updated by one or more identities.') +_(' If the dynamic update request message is signed (that is, it includes either a TSIG or SIG(0) record), the identity of the signer can be determined.') +_('\nRules are specified in the update-policy zone option, and are only meaningful for master zones.') +_(' When the update-policy statement is present, it is a configuration error for the allow-update statement to be present.') +_(' The update-policy statement only examines the signer of a message; the source address is not relevant.') ,
      'zone': True,
      'zone-only': True,
      'zone-types': ['master','stub','delegation-only']
    },
    'allow-recursion':
    { 'name': _('Allow Recursion'),
      'type': DNS.ACL,
      'desc': _('Specifies which hosts are allowed to make recursive queries through this server.') +_(' If not specified, the default is to allow recursive queries from all hosts.') +_(' Note that disallowing recursive queries for a host does not prevent the host from retrieving data that is already in the server\'s cache.') ,
    },
    'allow-update-forwarding':
    { 'name': _('Allow Update Forwarding'),
      'type': DNS.ACL,
      'zone': True,
      'zone-only': True,
      'desc': _('Specifies which hosts are allowed to submit Dynamic DNS updates to slave zones to be forwarded to the master.') +_(' The default is { none; }, which means that no update forwarding will be performed.') +_(' To enable update forwarding, specify allow-update-forwarding { any; };.') +_(' Specifying values other than { none; } or { any; } is usually counterproductive, since the responsibility for update access control should rest with the master server, not the slaves.') +_('\n Note that enabling the update forwarding feature on a slave server may expose master servers relying on insecure IP address based access control to attacks.') ,
      'zone-types': ['slave']
    },
    'database':
    { 'name': _('Database Driver'),
      'type': str ,
      'default': 'rbt',
      'zone': True,
      'zone-only': True,
      'desc': _('Specify the type of database to be used for storing the zone data.') +_(' The string following the database keyword is interpreted as a list of whitespace-delimited words.') +_(' The first word identifies the database type, and any subsequent words are passed as arguments to the database to be interpreted in a way specific to the database type.') +_('\nThe default is "rbt", BIND 9\'s native in-memory red-black-tree database.') +_(' This database does not take arguments.') +_(' Other values are possible if additional database drivers have been linked into the server.') +_(' Some sample drivers are included with the distribution but none are linked in by default.') ,
      'zone-types': ['master','slave','forward','stub','hint','delegation-only']
    },
    'allow-v6-synthesis':
    { 'name': _('Allow IPv6 Synthesis'),
      'type': DNS.ACL,
      'desc': _('Specifies which hosts are to receive synthetic responses to IPv6 queries.') +_('\nMany existing stub resolvers support IPv6 DNS lookups as defined in RFC1886, using AAAA records for forward lookups and "nibble labels" in the IP6.INT domain for reverse lookups, but do not support RFC2874-style lookups (using A6 records and binary labels in the IP6.ARPA domain).') +'\n' +_('For those who wish to continue to use such stub resolvers rather than switching to the BIND 9 lightweight resolver, BIND 9 provides a way to automatically convert RFC1886-style lookups into RFC2874-style lookups and return the results as "synthetic" AAAA and PTR records.') +_(' This feature is disabled by default and can be enabled on a per-client basis by adding a allow-v6-synthesis { address_match_list }; clause to the options or view statement.') +_(' When it is enabled, recursive AAAA queries cause the server to first try an A6 lookup and if that fails, AAAA lookups are attempted.') +_(' No matter which one succeeds, the results are returned as a set of synthetic AAAA records.') +_(' Similarly, recursive PTR queries in IP6.INT will cause a lookup in IP6.ARPA using binary labels, and if that fails, another lookup in IP6.INT.') +_(' The results are returned as a synthetic PTR record in IP6.INT.') +_('The synthetic records have a TTL of zero.') +_(' DNSSEC validation of synthetic responses is not currently supported; therefore responses containing synthetic RRs will not have the AD flag set.') +_('\n<b>Note</b>: allow-v6-synthesis is only performed for clients that are supplied recursive service.') ,
    },
    'blackhole':
    { 'name': _('Black Hole'),
      'type': DNS.ACL,
      'desc': _('Specifies a list of addresses that the server will not accept queries from or use to resolve a query.') +_(' Queries from these addresses will not be responded to.') +_(' The default is none.')
    },
    'listen-on':
    { 'name': _('IPv4 Port and Interfaces to Listen On'),
      'attrs':
      [ { 'port':
          { 'name': _('Port'),
            'type': int,
            'range': [0,65535],
            'syntax': True,
            'default': 53
          }
        },
        { 'interfaces':
          { 'name': _('Interface Addresses'),
            'type': DNS.ACL,
            'required':1,
            'default':'any',
            'values':[_('IPv4 Address'),_('None'),_('Any')]
          }
        },
      ],
      'desc': _('The port and a list IPv4 addresses on which named will listen for incoming queries can be specified with this option.') +_(' The name server will only receive queries directed to this list of addresses with the specified port.') +_('  The list of addresses must have IPv4 interfaces configured on the name server host machine.') +_(' The default is to listen on  all configured IPv4 interfaces with port #53.') +_(' Multiple <b>Listen On</b> options may be configured with different ports.')
    },
    'listen-on-v6':
    { 'name': 'IPv6 Port and Interfaces to listen on',
      'attrs':
      [ { 'port':
          { 'name': _('Port'),
            'type': int,
            'range': [0,65535],
            'syntax': True,
            'default': 53
          }
        },
        { 'interfaces' :
          { 'name': _('interfaces'),
            'type': DNS.ACL,
            'required':1,
            'default':'any',
            'values': [_('Any'), _('None')],
          }
        }
      ],
      'desc': _('This option is used to specify the ports on which the server will listen for incoming queries sent using IPv6.') +_('\nThe server does not bind a separate socket to each IPv6 interface address as it does for IPv4.') +_(' Instead, it always listens on the IPv6 wildcard address.') +_(' Therefore, the only values allowed for the address_match_list argument to the listen-on-v6 statement are "Any" and "None".') +_(' Multiple listen-on-v6 options can be used to listen on multiple ports.')
    }, 
    'query-source':
    { 'name':_('Query Source'),
      'attrs':
      [ { 'address':
          { 'name': _('Address'),
            'type': DNS.IPV4.Address,
            'syntax': True,
            'default': Lookup.localhost.addr
          }
        },
        { 'port':
          { 'name': _('Port'),
            'type': int,
            'range': [0,65535],
            'syntax': True,
            'default': 53
          }
        }
      ],            
      'desc': _('If the server doesn\'t know the answer to a question, it will query other nameservers.') +_(' <b>Query Source</b> specifies the address and port used for such queries.') +_(' If address is * or is omitted, a wildcard IP address (INADDR_ANY) will be used.') +_(' If port is * or is omitted, a random unprivileged port will be used.') +_(' <b>Query Source</b> may be especially useful if your nameserver is behind a firewall that can only unblock certain UDP ports.') +_('\n<b>Note</b>: The address specified in the query-source option is used for both UDP and TCP queries, but the port applies only to UDP queries.') +_(' TCP queries always use a random unprivileged port.')
    },
    'query-source-v6':
    { 'name': _('IPv6 Query Source'),
      'attrs':
      [ 
        { 'address':
          { 'name': _('Address'),
            'type': DNS.IPV6.Address,
            'syntax': True
          }
        },
        { 'port':
          { 'name': _('Port'),
            'type': int,
            'range': [0,65535],
            'syntax': True,
            'default': 53
          }
        }
      ],            
      'desc': _('If the server doesn\'t know the answer to a question, it will query other nameservers.') +_(' <b>Query Source IPv6</b> specifies the address and port used for such queries sent using IPv6.') +_(' If address is * or is omitted, a wildcard IP address (INADDR_ANY) will be used.') +_(' If port is * or is omitted, a random unprivileged port will be used.') +_(' <b>Query Source IPv6</b> may be especially useful if your nameserver is behind a firewall that can only unblock certain UDP ports.') +_('\n<b>Note</b>: The address specified in the query-source option is used for both UDP and TCP queries, but the port applies only to UDP queries.') +_(' TCP queries always use a random unprivileged port.') ,
    },
    'max-transfer-time-in' :
    { 'name': _('Maximum transfer time in'),
      'type': DNS.TTL,
      'conversion': 'M',
      'desc': _('Inbound zone transfers taking longer than this time will be terminated.') ,
      'zone': True,
      'zone-types': ['slave']
    },
    'max-transfer-time-out' :
    { 'name': _('Maximum transfer time out'),
      'type': DNS.TTL,
      'conversion': 'M',
      'desc': _('Outbound zone transfers taking longer than this time will be terminated.') ,
      'zone': True,
      'zone-types': ['master','stub','delegation-only']
    },
    'max-transfer-idle-in' :
    { 'name': _('Maximum transfer idle time in'),
      'type': DNS.TTL,
      'conversion': 'M',
      'desc': _('Inbound zone transfers making no progress for this length of time will be terminated.') ,
      'zone': True,
      'zone-types': ['slave']
    },
    'max-transfer-idle-out' :
    { 'name': _('Maximum transfer idle time out'),
      'type': DNS.TTL,
      'conversion': 'M',
      'desc': _('Outbound zone transfers making no progress for this length of time will be terminated.') ,
      'zone': True,
      'zone-types': ['master','stub','delegation-only']
    },
    'tcp-clients' :
    { 'name': _('Maximum number of TCP clients'),
      'type': int,
      'default': 100,
      'desc': _('No more than this number of simultaneous incoming TCP connections will be accepted.') ,
    },    
    'recursive-clients' :
    { 'name': _('Recursive Clients'),
      'type': int,
      'default': 1000,
      'desc': _('The maximum number of simultaneous recursive lookups the server will perform on behalf of clients.') +_(' The default is 1000.') +_(' Because each recursing client uses a fair bit of memory, on the order of 20 kilobytes, the value of the recursive-clients option may have to be decreased on hosts with limited memory.') ,
    },
    'serial-query-rate' :
    { 'name': _('Serial Query Rate'),
      'type': int,
      'desc': _('Slave servers will periodically query master servers to find out if zone serial numbers have changed.') +_(' Each such query uses a minute amount of the slave server\'s network bandwidth.') +_(' To limit the amount of bandwidth used, BIND 9 limits the rate at which queries are sent.') +_(' The value of the serial-query-rate option, an integer, is the maximum number of queries sent per second.') +_(' The default is 20.') ,
    },
    'serial-queries' :
    { 'name': _('Serial Queries'),
      'type': int,
      'deprecated': True,
      'desc': _('In BIND 8, the serial-queries option set the maximum number of concurrent serial number queries allowed to be outstanding at any given time.') +_(' BIND 9 does not limit the number of outstanding serial queries and ignores the serial-queries option.') +_(' Instead, it limits the rate at which the queries are sent as defined using the serial-query-rate option.')
    },
    'transfer-format':
    {  'name': _('Transfer Format'),
       'type': DNS.Radio,
       'values': [ 'one-answer', 'many-answers' ],
       'default': 1,
       'desc': _('Zone transfers can be sent using two different formats, <tt>one-answer</tt> and <tt>many-answers</tt>.') +_(' The <b>Transfer Format</b> option is used on the master server to determine which format it sends.') +_(' <tt>one-answer</tt> uses one DNS message per resource record transferred.') +_(' <tt>many-answers</tt> packs as many resource records as possible into a message.') +_(' <tt>many-answers</tt> is more efficient, but is only supported by relatively new slave servers, such as BIND 9, BIND 8.x and patched versions of BIND 4.9.5 .') +_(' The default is <tt>many-answers</tt>.') +_(' <b>Transfer Format</b> may be overridden on a per-server basis by using the server statement.') ,
       'server':True       
    },
    'transfers-in' :
    {  'name': _('Inbound Transfer Limit'),
       'type': int,
       'desc': _('The maximum number of inbound zone transfers that can be running concurrently.') +_(' The default value is 10.') +_(' Increasing transfers-in may speed up the convergence of slave zones, but it also may increase the load on the local system.') ,
    },
    'transfers-out' :
    {  'name': _('Outbound Transfer Limit'),
       'type': int,
       'desc': _('The maximum number of outbound zone transfers that can be running concurrently.') +_(' Zone transfer requests in excess of the limit will be refused.') +_(' The default value is 10.') ,
    },
    'transfers-per-ns' :
    {  'name': _('Transfers per Name Server Limit'),
       'type': int,
       'desc': _('The maximum number of inbound zone transfers that can be concurrently transferring from a given remote nameserver.') +_(' The default value is 2.') +_(' Increasing transfers-per-ns may speed up the convergence of slave zones, but it also may increase the load on the remote nameserver.') +_(' transfers-per-ns may be overridden on a per-server basis by using the transfers phrase of the server statement.')
    },
    'transfer-source':
    {  'name': _('IPV4 Transfer Source'),
       'attrs':
       [ { 'address':
           { 'name': _('Address'),
             'type': DNS.IPV4.Address,
             'syntax': True,
             'default': Lookup.localhost.addr
           }
         },
         { 'port':
          { 'name': _('Port'),
            'type': int,
            'range': [0,65535],
            'syntax': True,
            'default': 53
          }
         }
       ],            
       'desc': _('This option determines which local address will be bound to IPv4 TCP connections used to fetch zones transferred inbound by the server.') +_(' It also determines the source IPv4 address, and optionally the UDP port, used for the refresh queries and forwarded dynamic updates.') +_(' If not set, it defaults to a system controlled value which will usually be the address of the interface "closest to" the remote end.') +_(' This address must appear in the remote end\'s allow-transfer option for the zone being transferred, if one is specified.') +_(' This statement sets the transfer-source for all zones, but can be overridden on a per-view or per-zone basis by including a transfer-source statement within the view or zone block in the configuration file.') ,
       'zone': True,
       'zone-types': ['slave']       
    },
    'transfer-source-v6':
    {  'name': _('IPV6 Transfer Source'),
       'attrs':
       [ 
        { 'address':
          { 'name': _('Address'),
            'type': DNS.IPV6.Address,
            'syntax': True
          }
        },
        { 'port':
          { 'name': _('Port'),
            'type': int,
            'range': [0,65535],
            'syntax': True,
            'default': 53
          }
        }        
       ],            
       'desc': _('This option determines which local address will be bound to IPv6 TCP connections used to fetch zones transferred inbound by the server.') +_(' It also determines the source IPv6 address, and optionally the UDP port, used for the refresh queries and forwarded dynamic updates.') +_(' If not set, it defaults to a system controlled value which will usually be the address of the interface "closest to" the remote end.') +_(' This address must appear in the remote end\'s allow-transfer option for the zone being transferred, if one is specified.') +_(' This statement sets the transfer-source for all zones, but can be overridden on a per-view or per-zone basis by including a transfer-source statement within the view or zone block in the configuration file.') ,
       'zone': True,
       'zone-types': ['slave']       
    },
    'notify-source':
    { 'name': _('IPV4 Notify Source'),
       'attrs':
       [ 
        { 'address':
          { 'name': _('Address'),
            'type': DNS.IPV4.Address,
            'syntax': True,
            'default': Lookup.localhost.addr
          }
        },
        { 'port':
          { 'name': _('Port'),
            'type': int,
            'range': [0,65535],
            'syntax': True,
            'default': 53
          }
        }
       ],            
      'desc': _('This option determines which local source IPv4 address, and optionally UDP port, will be used to send NOTIFY messages.') +_(' This address must appear in the slave server\'s masters zone clause or in an allow-notify clause.') +_(' This statement sets the notify-source for all zones, but can be overridden on a per-zone / per-view basis by including a notify-source statement within the zone or view block in the configuration file.') ,
      'zone': True,
      'zone-types': ['master','stub','delegation-only']
    },          
    'notify-source-v6':
    { 'name': _('IPv6 Notify Source'),
       'attrs':
       [ 
        { 'address':
          { 'name': _('Address'),
            'type': DNS.IPV6.Address,
            'syntax': True
          }
        },
        { 'port':
          { 'name': _('Port'),
            'type': int,
            'range': [0,65535],
            'syntax': True,
            'default': 53
          }
        }
       ],            
      'desc': _('This option determines which local source IPv6 address, and optionally UDP port, will be used to send NOTIFY messages.') +_(' This address must appear in the slave server\'s masters zone clause or in an allow-notify-v6 clause.') +_(' This statement sets the notify-source for all zones, but can be overridden on a per-zone / per-view basis by including a notify-source statement within the zone or view block in the configuration file.') ,
      'zone': True,
      'zone-types': ['master','stub','delegation-only']
    },          
    'also-notify':
    { 'name': _('Also Notify'),
      'type': DNS.AddressList,
      'desc': _('Defines a global list of IP addresses of name servers that are also sent NOTIFY messages whenever a fresh copy of the zone is loaded, in addition to the servers listed in the zone\'s NS records.') +_(' This helps to ensure that copies of the zones will quickly converge on stealth servers.') +_(' If an also-notify list is given in a zone statement, it will override the options also-notify statement.') +_(' When a zone notify statement is set to no, the IP addresses in the global also-notify list will not be sent NOTIFY messages for that zone.') +_(' The default is the empty list (no global notification list).') ,
      'zone': True,
      'zone-types': ['master','stub','delegation-only']
    },
    'max-ixfr-log-size':
    {  'name': _('max_ixfr_log_size'),
       'deprecated':True,
       'type': DNS.Size,
       'desc': _('This option is obsolete; it is accepted and ignored for BIND 8 compatibility.') ,
       'zone':True,
       'zone-types': ['master','slave','stub','delegation-only']       
    },
    'coresize':
    {  'name': _('Core File Size Limit'),
       'type': DNS.Size,
       'desc': _('The maximum size of a core dump file.') ,
    },
    'datasize':
    {  'name': _('Data Memory Size Limit'),
       'type': DNS.Size,
       'desc': _('The maximum amount of data memory the server may use.') +_(' This is a hard limit on server memory usage.') +_(' If the server attempts to allocate memory in excess of this limit, the allocation will fail, which may in turn leave the server unable to perform DNS service.') +_(' Therefore, this option is rarely useful as a way of limiting the amount of memory used by the server, but it can be used to raise an operating system data size limit that is too small by default.') +_(' If you wish to limit the amount of memory used by the server, use the max-cache-size and recursive-clients options instead.') ,
    },
    'files':
    {  'name': _('Number of Open Files Limit'),
       'type': DNS.Size,
       'desc': _('The maximum number of files the server may have open concurrently.') +_(' The default is unlimited.') ,
    },
    'stacksize':
    {  'name': _('Stack Memory Size Limit'),
       'type': DNS.Size,
       'desc': _('The maximum amount of stack memory the server may use.') +_(' The default is unlimited.') ,
    },
    'cleaning-interval' :
    {  'name': _('Cleaning Interval'),
       'type': DNS.TTL,
       'conversion': 'M',
       'desc': _('The server will remove expired resource records from the cache every cleaning-interval minutes.') +_(' The default is 60 minutes.') +_(' If set to 0, no periodic cleaning will occur.') ,
    },
    'heartbeat-interval':
    {  'name': _('Heartbeat Interval'),
       'type': DNS.TTL,
       'conversion': 'M',
       'desc': _('The server will perform zone maintenance tasks for all zones marked as dialup whenever this interval expires.') +_(' The default is 60 minutes.') +_(' Reasonable values are up to 1 day (1440 minutes).') +_(' If set to 0, no zone maintenance for these zones will occur.') ,
    },    
    'interface-interval':
    {  'name': _('Interface Interval'),
       'type': DNS.TTL,
       'conversion': 'M',
       'desc': _('The server will scan the network interface list every interface-interval minutes.') +_(' The default is 60 minutes.') +_(' If set to 0, interface scanning will only occur when the configuration file is loaded.') +_(' After the scan, listeners will be started on any new interfaces (provided they are allowed by the listen-on configuration).') +_(' Listeners on interfaces that have gone away will be cleaned up.') ,
    },
    'statistics-interval' :
    {  'name': _('Statistics Interval'),
       'type': DNS.TTL,
       'conversion': 'M',
       'desc': _('Nameserver statistics will be logged every statistics-interval minutes.') +_(' The default is 60.') +_(' If set to 0, no statistics will be logged.') +_('\n<b>Note</b>: Not yet implemented in BIND9.') ,
       'unimplemented':True
    },
    'topology':
    {  'name': _('Topology'),
       'type': DNS.ACL,
       'desc': _('All other things being equal, when the server chooses a nameserver to query from a list of nameservers, it prefers the one that is topologically closest to itself.') +_(' The topology statement takes an address_match_list and interprets it in a special way.') +_(' Each top-level list element is assigned a distance.') +_(' Non-negated elements get a distance based on their position in the list, where the closer the match is to the start of the list, the shorter the distance is between it and the server.') +_(' A negated match will be assigned the maximum distance from the server.') +_(' If there is no match, the address will get a distance which is further than any non-negated list element, and closer than any negated element.') ,
       'unimplemented':True
    },     
    'sortlist':
    {  'name': _('Sort List'),
       'type': DNS.SortList,
       'desc': _('The response to a DNS query may consist of multiple resource records (RRs) forming a resource records set (RRset).') +_(' The name server will normally return the RRs within the RRset in an indeterminate order.') +_(' The client resolver code should rearrange the RRs as appropriate, that is, using any addresses on the local net in preference to other addresses.') +_(' However, not all resolvers can do this or are correctly configured.') +_(' When a client is using a local server the sorting can be performed in the server, based on the client\'s address.') +_(' This only requires configuring the nameservers, not all the clients.') +_('\nThe sortlist statement takes an address_match_list and interprets it even more specifically than the topology statement does.') +_(' Each top level statement in the sortlist must itself be an explicit address_match_list with one or two elements.') +_(' The first element (which may be an IP address, an IP prefix, an ACL name or a nested address_match_list) of each top level list is checked against the source address of the query until a match is found.') +_('\nOnce the source address of the query has been matched, if the top level statement contains only one element, the actual primitive element that matched the source address is used to select the address in the response to move to the beginning of the response.') +_(' If the statement is a list of two elements, then the second element is treated the same as the address_match_list in a topology statement.') +_(' Each top level element is assigned a distance and the address in the response with the minimum distance is moved to the beginning of the response.')
   },
   'rrset-order':
   { 'name': _('Resource Record Set Order'),
     'type': DNS.Order,
     'desc': _('When multiple records are returned in an answer it may be useful to configure the order of the records placed into the response.') +_(' The rrset-order statement permits configuration of the ordering of the records in a multiple record response.') ,
     'version': '9.3.0',
     'todo':True
   },   
   'lame-ttl' :
   { 'name': _('Lame Delegation Cache Time To Live'),
     'type': DNS.TTL,
     'conversion': 'S',
     'default': '10S',
     'max': '600S',
     'desc': _('Sets the number of seconds to cache a lame server indication.') +_(' 0 disables caching.') +_(' (This is NOT recommended.') +_(' Default is 600 (10 minutes).') +_(' Maximum value is 1800 (30 minutes).') ,
   },
   'max-ncache-ttl' :
   { 'name': _('Maximum Negative Cache Time To Live'),
     'type': DNS.TTL,
     'conversion': 'S',
     'max': '7D',
     'desc': _('To reduce network traffic and increase performance the server stores negative answers.') +_(' max-ncache-ttl is used to set a maximum retention time for these answers in the server in seconds.') +_(' The default max-ncache-ttl is 10800 seconds (3 hours).') +_(' max-ncache-ttl cannot exceed 7 days and will be silently truncated to 7 days if set to a greater value.') ,
   },
   'max-cache-ttl' :
   {  'name': _('Maximum Cache Time To Live'),
        'type': DNS.TTL,
        'desc': _('max-cache-ttl sets the maximum time for which the server will cache ordinary (positive) answers.') +_(' The default is one week (7 days).') ,
   },
   'sig-validity-interval':
   {  'name': _('Signature Validity Interval'),
      'type': DNS.TTL,
      'default':'30D',
      'desc': _('Specifies the number of days into the future when DNSSEC signatures automatically generated as a result of dynamic updates will expire.') +_(' The default is 30 days.') +_(' The signature inception time is unconditionally set to one hour before the current time to allow for a limited amount of clock skew.') ,
      'zone': True,
      'zone-types': ['master','stub','delegation-only']
   },
   'min-roots' :
   {  'name':_('Minimum Number of Root Servers'),
      'type': int,
      'default': 2,
      'desc': _('The minimum number of root servers that is required for a request for the root servers to be accepted.') +_(' Default is 2.') +_(' Note: Not yet implemented in BIND9.') ,
      'unimplemented':True
   },
   'use-ixfr':
   {  'name':_('Use Incremental Zone Transfer'),
      'deprecated':True,
      'type': DNS.Boolean,
      'values': [ _('Disable'), _('Enable') ],
      'default': 1,
      'desc': _('This option is obsolete.') +_(' If you need to disable IXFR to a particular server or servers see the information on the <b>Provide Incremental Zone Transfer</b> option.') ,
   },
   'provide-ixfr' :
   {  'name':_('Provide Incremental Zone Transfer'),
      'type': DNS.Boolean,
      'values': [_('Disable'),_('Enable')],
      'default':1,
      'desc': _('This option determines whether the local server, acting as master, will respond with an incremental zone transfer when a remote slave server requests it.') +_(' If <tt>Enabled</tt>, incremental transfer will be provided whenever possible.') +_(' If <tt>Disabled</tt>, all transfers to the remote slave server will be nonincremental.') ,
      'server':True
   },
   'request-ixfr' :
   {  'name':_('Request Incremental Zone Transfer'),
      'type': DNS.Boolean,
      'default':1,
      'values': [_('Disable'),_('Enable')],
      'desc': _('This option determines whether the local server, acting as a slave, will request incremental zone transfers from a remote master server.') ,
      'server':True
   },       
   'treat-cr-as-space':
   {  'name':_('Treat CR as Space'),
      'deprecated':True,
      'type': DNS.Boolean,
      'default':1,
      'values': [_('Disable'),_('Enable')],
      'desc': _('This option was used in BIND 8 to make the server treat carriage return ("\\r") characters the same way as a space or tab character, to facilitate loading of zone files on a UNIX system that were generated on an NT or DOS machine.') +_(' In BIND 9, both UNIX "\\n" and NT/DOS "\\r\\n" newlines are always accepted, and the option is ignored.') ,
   },
   'min-refresh-time':
   {  'name': _('Minimum Refresh Time'),
      'type': DNS.TTL,
      'desc': _('The minimum time that must elapse before a zone refresh (SOA record query) is attempted.') ,
      'zone': True,
      'zone-types': ['slave','forward']
   },   
   'max-refresh-time':
   {  'name': _('Maximum Refresh Time'),
      'type': DNS.TTL,
      'desc': _('The maximum time that can elapse before a zone refresh (SOA record query) is attempted.') ,
      'zone': True,
      'zone-types': ['slave','forward']
   },
   'min-retry-time':
   {  'name': _('Minimum Refresh Time'),
      'type': DNS.TTL,
      'desc': _('The minimum time that must elapse before a failed zone refresh (SOA record query) or zone transfer is re-attempted.') ,
      'zone': True,
      'zone-types': ['slave','forward']
   },   
   'max-retry-time':
   {  'name': _('Maximum Refresh Time'),
      'type': DNS.TTL,
      'desc': _('The maximum time that can elapse before a failed zone refresh (SOA record query) or zone transfer is re-attempted.') ,
      'zone': True,
      'zone-types': ['slave','forward']
   },     
   'additional-from-auth':
   {  'name': _('Obtain Additional Data from Authoritative Zones'),
      'type': DNS.Boolean,
      'values': [_('Disable'),_('Enable')],
      'default': 1,
      'desc': _('Append additional data to recursive queries when it is available from authoritative zone.')
   },
   'additional-from-cache':
   {  'name': _('Obtain Additional Data from Cache'),
      'type': DNS.Boolean,
      'values': [_('Disable'),_('Enable')],
      'default': 1,
      'desc': _('Append additional data to recursive queries when it is available from the cache.')
   },
   'match-mapped-addresses':
    {  'name': _('Match IPV4 Mapped IPV6 Addresses'),
       'type': DNS.Boolean,
       'values': [_('Disable'),_('Enable')],
       'default': 0,
       'desc': _('If <tt>Enabled</tt>, then an IPv4-mapped IPv6 address will match any address match list entries that match the corresponding IPv4 address.') +_(' Enabling this option is sometimes useful on IPv6-enabled Linux systems, to work around a kernel quirk that causes IPv4 TCP connections such as zone transfers to be accepted on an IPv6 socket using mapped addresses, causing address match lists designed for IPv4 to fail to match.') +_(' The use of this option for any other purpose is discouraged.')
   },
   'port':
    {  'name': _('Server Port'),
       'type': int,
       'default':53,       
       'range': [0,65535],
       'desc': _('The UDP/TCP port number the server uses for receiving and sending DNS protocol traffic.') +_(' The default is 53.') +_(' This option is mainly intended for server testing; a server using a port other than 53 will not be able to communicate with the global DNS.')
   },      
   'random-device':
   {  'name': _('Random Device Path'),
      'type': DNS.Path,
      'default': '/dev/random',
      'desc': _('The path of the random device special file to use if not /dev/random.') ,
   },
   'max-cache-size':
   {  'name': _('Maximum Cache Size'),
      'type': DNS.Size,
      'desc': _('The maximum allowed size of the cache.') ,
   },
   'root-delegation-only':
   {  'name': _('Root Delegation Only'),
      'attrs':
      [ { 'exclude' :
          { 'name': _('Excluded Domains'),
            'type': DNS.ACL,
            'syntax': True            
          }
        }
      ],
      'desc': _('Turn on enforcement of delegation-only in Top Level Domains and root zones with an optional exclude list.') ,
   },
   'match-clients':
   {  'name': _('View Clients'),
      'type': DNS.ACL,
      'desc': _('Defines the query client hosts source IP addresses this view will be applied to.') ,
      'view-only': True      
   },
   'match-destinations':
   {  'name': _('View Destinations'),
      'type': DNS.ACL,
      'desc': _('Defines the query client destination IP addresses this view will be applied to.') ,
      'view-only': True      
   },
   'match-recursive-only':
   {  'name': _('Recursive Clients Only'),
      'type': DNS.Boolean,
      'values': [_('Disable'),_('Enable')],
      'default':0,
      'desc': _('If enabled, ONLY recursive requests from matching clients will match this view.') ,
      'view-only': True      
   },
   'bogus':
   {  'name': _('Bogus Server'),
      'type': DNS.Boolean,
      'values': [_('Disable'),_('Enable')],
      'default': 0,
      'desc': _('If Enabled, this server is known to supply bad / untrustworthy responses to queries, and no queries are to be directed to it.') ,
      'server': True,
      'server-only': True
   },
   'edns':
   {  'name': _('Extended DNS Server'),
      'type': DNS.Boolean,
      'values': [_('Disable'),_('Enable')],
      'default': 1,
      'desc': _('If Enabled, this server is assumed to support RFC 2671 DNS Protocol Extensions (EDNS0).') ,
      'server': True,
      'server-only': True
   },
   'transfers':
   {  'name': _('Concurrent Transfers Limit'),
      'type': int,
      'desc': _('The maximum number of concurrent inbound zone transfers allowed from this server.') ,
      'server': True,
      'server-only': True
   },
   'keys':
   {  'name': _('Server Keys'),
      'type': DNS.ACL,
      'values': [ 'Named Key' ],
      'desc': _('The name of a key defined with the <b>Key</b> statement to be used for communication with this server.') ,
      'server': True,
      'server-only': True
   },
#  New 9.3.0 Options:
   'hostname':
   {  'name': _('Server Hostname'),
      'type': str,
      'desc': _('The hostname the server should report via a query of the name "hostname.bind" with type TXT, class CHAOS.') +_(' This defaults to the hostname of the machine hosting the name server as found by gethostname().') +_(' The primary purpose of such queries is to identify which of a group of anycast servers is actually answering your queries.') +_(' Specifying hostname none; disables processing of the queries.') ,
      'version':'9.3'
   },
   'server-id':
   {  'name': _('Server Identifier'),
      'type': str,
      'desc': _('The ID of the server should report via a query of the name "id.server" with type TXT, class CHAOS.') +_(' The primary purpose of such queries is to identify which of a group of anycast servers is actually answering your queries.') +_(' Specifying server-id none; disables processing of the queries.') +_(' Specifying server-id hostname; will cause named to use the hostname as found by gethostname().') +_(' The default server-id is none.') ,
      'version':'9.3'
   },
   'key-directory':
   {  'name': _('Key File Path'),
      'type': DNS.Path,
      'isDirectory':True,
      'isAbsolute':True,
      'desc': _('When performing dynamic update of secure zones, the directory where the public and private key files should be found, if different than the current working directory.') +_(' The directory specified must be an absolute path.') ,
      'version':'9.3',      
      'zone':True,
      'zone-types':['master','slave','forward','stub','delegation-only']
   },
   'flush-zones-on-shutdown':
   {  'name': _('Write Updated Zones on Abnormal Exit'),
      'type': DNS.Boolean,
      'values':[_('Disable'),_('Enable')],
      'default':0,
      'version':'9.3',
      'desc': _('When the nameserver exits without being shutdown by <tt>rndc(8)</tt>, but by being killed with the <tt>SIGTERM</tt> signal, if this option is <tt>Enable</tt>d, flush any pending zone writes from the in-memory database to disk, otherwise do not.') +_(' The default value of this option is <tt>Disable</tt>.') ,
   },
   'dnssec-enable':
   {  'name': _('Enable new secure DNS protocols'),
      'type': DNS.Boolean,
      'values':[_('Disable'),_('Enable')],
      'default':0,
      'version':'9.3',
      'desc': _('Enable DNSSEC support in named.') +_(' Unless set to <tt>Enable</tt> named behaves as if it does not support DNSSEC.') +_(' The default is <tt>Disable</tt>.') ,
   },
   'dnssec-lookaside':
   {  'name': _('DNSSEC Lookaside'),
      'attrs':[ { 'domain' :
                  {  'name': _('Domain'),
                     'type': DNS.Name
                  }
                },
                { 'trust-anchor' :
                  {  'name': _('Trust Anchor'),
                     'type': DNS.Name,
                     'syntax':True                     
                  }
                }
               ],
      'version':'9.3',
      'desc': _('When set dnssec-lookaside provides the validator with an alternate method to validate DNSKEY records at the top of a zone.') +_(' When a DNSKEY is at or below a domain specified by the deepest dnssec-lookaside, and the normal dnssec validation has left the key untrusted, the trust-anchor will be append to the key name and a DLV record will be looked up to see if it can validate the key.') +_(' If the DLV record validates a DNSKEY (similarly to the way a DS record does) the DNSKEY RRset is deemed to be trusted.') ,
   },
   'dnssec-must-be-secure':
   {  'name': _('Domain DNSSEC Validation'),
      'attrs':[ { 'domain' :
                  {  'name': _('Domain'),
                     'type': DNS.Name
                  }
                },
                { 'secure':
                  {  'name': _('Validate'),
                     'type': DNS.Boolean,
                     'values':[_('Enable'),_('Disable')],
                     'default':0
                  }
                }
              ],
      'version':'9.3',
      'desc': _('Specify domain hierarchies which must / may not be secure (signed and validated).') +_(' If <tt>Enable</tt>-d, then named will only accept answers if they are secure.') +_(' If <tt>Disable</tt>d then normal dnssec validation applies allowing for insecure answers to be accepted.') +_(' The specified domain must be under a trusted-key or dnssec-lookaside must be active.') ,
   },
   'disable-algorithms':
   {  'name': _('Disabled DNSSEC Algorithms'),
      'version':'9.3',
      'attrs':[ { 'domain' :
                  {  'name': _('Domain'),
                     'type': DNS.Name
                  }
                },
                { 'algorithms':
                  {  'name': _('Algorithms'),
                     'type': DNS.AlgorithmList
                  }
                }
              ],
      'desc': _('Disable the specified DNSSEC algorithms at and below the specified name.') +_(' Multiple disable-algorithms statements are allowed.') +_(' Only the most specific will be applied.') ,
     'todo':True
   },
   'preferred-glue':
   {  'name': _('Preferred Glue Type'),
      'type': DNS.Radio,
      'values': ['NONE', 'A','AAAA'],
      'default': 0,
      'version': '9.3',
      'desc': _('If specified the listed type (A or AAAA) will be emitted before other glue in the additional section of a query response.') +_(' The default is not to preference any type (NONE).') ,
   },
   'dual-stack-servers':
   {  'name': _('Dual Stack Servers'),
      'attrs': [ { 'port':
                   {  'name': _('Port'),
                      'syntax': True,
                      'type': int,
                      'range':[0,65535],
                      'default':53
                   },
                   'servers':
                   {  'name': _('Servers'),
                      'type': DNS.AddressList,
                      'enable-domain-names':True,
                   }
                 }
               ],
      'version': '9.3',
      'desc': _('Specifies host names / addresses of machines with access to both IPv4 and IPv6 transports.') +_(' If a hostname is used the server must be able to resolve the name using only the transport it has.') +_(' If the machine is dual stacked then the dual-stack-servers have no effect unless access to a transport has been disabled on the command line, (e.g. with the named "-4" or "-6" options).') ,
   },
   'avoid-v4-udp-ports':
   {  'name': _('IPv4 Ports to Avoid'),
      'type': DNS.PortList,
      'version': '9.3',
      'desc': _('Specify a list of IPv4 ports that will not be used as system assigned source ports for UDP sockets.') +_(' This list prevents named from choosing as its random IPv4 source port a port that is blocked by your firewall.') +_(' If a query went out with such a source port, the answer would not get by the firewall and the name server would have to query again.') ,
     'todo':True
   },
   'avoid-v6-udp-ports':
   {  'name': _('IPv6 Ports to Avoid'),
      'type': DNS.PortList,
      'version': '9.3',
      'desc': _('Specify a list of IPv6 ports that will not be used as system assigned source ports for UDP sockets.') +_(' This list prevents named from choosing as its random IPv4 source port a port that is blocked by your firewall.') +_(' If a query went out with such a source port, the answer would not get by the firewall and the name server would have to query again.') ,
     'todo':True
   },
   'tcp-listen-queue':
   {  'name': _('TCP listen queue'),
      'type': int,
      'range':[3,1024],
      'version':'9.3',
      'desc': _('The listen queue depth, or number of client connections that can be queued pending acceptance.') +_(' The default and minimum is 3.') +_(' If the kernel supports the accept filter "dataready" this also controls how many TCP connections that will be queued in kernel space waiting for some data before being passed to accept.') +_(' Values less than 3 will be silently raised.') +_(' This value cannot exceed the kernel <b>tcp_max_syn_backlog</b> sysctl(8) parameter.') ,
   },
   'alt-transfer-source':
    {  'name': _('IPV4 Alternative Transfer Source'),
       'attrs':
       [ { 'address':
           { 'name': _('Address'),
             'type': DNS.IPV4.Address,
             'syntax': True,
             'default': Lookup.localhost.addr
           }
         },
         { 'port':
          { 'name': _('Port'),
            'type': int,
            'range': [0,65535],
            'syntax': True,
            'default': 53
          }
         }
       ],            
       'desc': _('An alternative IPv4 transfer source if the one listed in transfer-source fails and use-alt-transfer-source is set.') ,
       'zone': True,
       'zone-types': ['slave'],
       'version':'9.3'
    },
    'alt-transfer-source-v6':
    {  'name': _('IPV6 Alternative Transfer Source'),
       'attrs':
       [ 
        { 'address':
          { 'name': _('Address'),
            'type': DNS.IPV6.Address,
            'syntax': True
          }
        },
        { 'port':
          { 'name': _('Port'),
            'type': int,
            'range': [0,65535],
            'syntax': True,
            'default': 53
          }
        }        
       ],
       'desc': _('An alternative IPv6 transfer source if the one listed in transfer-source-v6 fails and use-alt-transfer-source is set.') ,
       'zone': True,
       'zone-types': ['slave'],
       'version':'9.3'
    },
    'use-alt-transfer-source':
    {  'name': _('Enable use of the alternative transfer sources'),
       'type': DNS.Boolean,
       'values': [ _('Disable'), _('Enable')],
       'default': 0,
       'version':'9.3',
       'zone': True,
       'zone-types': ['slave'],
       'desc': _('Use the alternate transfer sources or not.') +_(' If views are specified this defaults to no otherwise it defaults to yes (for BIND 8 compatibility).') ,
    },
    'max-journal-size':
    {  'name': _('Maximum journal file size'),
       'type': DNS.Size,
       'default':'unlimited',
       'version':'9.3',
       'desc': _('Sets a maximum size for each journal file.') +_(' When the journal file approaches the specified size, some of the oldest transactions in the journal will be automatically removed.') +_(' The default is unlimited.')
    },
    'edns-udp-size':
    {  'name': _('EDNS UDP Packet Size'),
       'type': int,
       'range': [512,4096],
       'desc': _('This sets the advertised EDNS UDP packet size.') +_(' Valid values are 512 to 4096 (values outside this range will be silently adjusted).') +_(' The default value is 4096.') +_(' The usual reason for setting edns-udp-size to a non default value is to get UDP answers to pass through broken firewalls that block fragmented packets and/or block UDP packets that are greater than 512 bytes.') ,
       'version':'9.3'
    },
    'querylog':
    {  'name': _('Enable Query Logging'),
       'type': DNS.Boolean,
       'values': [ _('Disable'), _('Enable')],
       'default': 0,
       'version': '9.3',
       'desc': _('Specify whether query logging should be enabled when named starts.') +_(' If querylog is not specified then the query logging is determined by the presence of the logging category queries.') ,
    },
    'multi-master':
    {  'name': _('Multiple Masters'),
       'type': DNS.Boolean,
       'values': [ _('Disable'), _('Enable')],
       'default': 0,
       'version': '9.3',
       'desc': _('This should be set when you have multiple masters for a zone and the addresses refer to different machines.') +_(' If <tt>Enabled</tt>, named will not log when the serial number on the master is less than what named currently has.') +_(' The default is no.') ,
       'zone':True,
       'zone-only':True,
       'zone-types':['slave']
    }
}
      
