#!/usr/bin/python
#
# BIND Resource Record classes
#
# Jason Vas Dias <jvdias@redhat.com>, Red Hat Inc. Sep 2004

import string
import types
import DNS

WANT_ZONE=-1
WANT_TOKENS=-2
WANT_ZONE_AND_TOKENS=-3

class RR:
    zone=None
    name=None
    clas=None
    ttl=None
    attrs=[]
    tokens=[]
    offsets=[]
    file=None
    continuation=False
    rdata_token=-1
    index=-1
    classes=[ 'IN', 'HS', 'CH' ]
    classes_desc=[_('Internet'),'HESIOD','CHAOS']
    def has_data(self):
        return hasattr(self,'new_record') or ( (len(self.attrs)>0) and (len(self.attrs) <= len(self.tokens)) )
    
    def parse(self):
        if self.has_data():            
            tokens = self.tokens[self.rdata_token:]
            tokno  = self.rdata_token
            for attr in self.attrs:
                token = tokens.pop(0)
                if (len(attr)==3):
                    if( attr[2] > 0 ):
                        setattr(self, attr[0], attr[1](self.tokens[attr[2]-1:]))
                    elif(attr[2] == WANT_ZONE):                    
                        setattr(self, attr[0], attr[1](token, self.zone))
                    elif(attr[2] == WANT_TOKENS):
                        setattr(self, attr[0], attr[1](self.tokens[tokno:]))
                    elif(attr[2] == WANT_ZONE_AND_TOKENS):
                        setattr(self, attr[0], attr[1](self.zone, self.tokens[tokno:]))
                        break
                else:
                    setattr(self, attr[0], attr[1](token))
                tokno += 1

    def type_token(s):
        if( len(self.tokens) == 0):
            return -1
        typ = s.Type().upper()
        i = 0
        for t in self.tokens:
            if t.upper == typ:
                return i;
            i+=1
        return -1
    
    def out(self, origin=None):
        output=''
        last_name = None
        if not self.has_data():
            return output
        if origin == None:
            origin = self.zone.zone_origin
        tokens=self.tokens[:]
        offsets=self.offsets[:]
        if hasattr(self,'directive') :
            output+=tokens.pop(0)
            if(len(offsets)>1):
                offset=offsets.pop(0)
                output+=self.zone.contents[offset[1]:offsets[0][0]]
            else:
                output+="\t"
            output+=self.value()
            if(len(offsets)>1):
                offset=offsets.pop(0)
                output+=self.zone.contents[offset[1]:offsets[0][0]]
            else:
                output+="\t"            
            return output
        if( self.name != None ):
            output+=self.name.relative_value(origin)
            if len(tokens):
                tokens.pop(0)
            if(len(offsets)>1):
                offset=offsets.pop(0)
                output+=self.zone.contents[offset[1]:offsets[0][0]]
            else:
                output+="\t"
        else:
            output+="\t"
        clas=str(self.__class__).split('.')[-1]
        if( (self.ttl != None) or (self.clas != None)):
            if( len(offsets) ):
                prefix_tokens=tokens[:min(3,len(tokens))]
                ttlOutput=False
                clasOutput=False
                for token in prefix_tokens:
                    if (token.upper() == clas) or (token.upper() in RR.classes):
                        if (token.upper() == clas):
                            if (self.ttl != None) and (ttlOutput == False):
                                output+=str(self.ttl)+"\t"
                            if (self.clas != None) and (clasOutput == False):
                                output+=str(self.clas)+"\t"                                
                        else:
                            clasOutput = True
                        output+=tokens.pop(0)
                        if(len(offsets)>1):
                            offset=offsets.pop(0)
                            output+=self.zone.contents[offset[1]:offsets[0][0]]
                        else:
                            output+="\t"
                        if (token == clas):
                            break
                    elif (self.ttl != None) and (ttlOutput == False):
                        output+=str(self.ttl)
                        tokens.pop(0)
                        if( len(offsets) > 1 ):
                            offset=offsets.pop(0)                        
                            output+=self.zone.contents[offset[1]:offsets[0][0]]
                        else:
                            output+="\t"
                        ttlOutput=True
            else:
                if (self.clas != None):
                    output+=  self.clas + "\t"
                if (self.ttl != None):
                    output+=  str(self.ttl) + "\t"
                output+= clas + "\t"
        else:
            if len(tokens) > 0:
                output += tokens.pop(0)
            else:
                output += self.Type()
            if len(offsets) > 1:
                offset=offsets.pop(0)                        
                output+=self.zone.contents[offset[1]:offsets[0][0]]
            else:
                output+="\t"
        wasOffsets=(len(offsets)>=1)
        for attr in self.attrs:
            if (attr[1] == int) or (attr[1] == str):
                v = str(getattr(self,attr[0]))
            else:
                v=str(getattr(self,attr[0]).value())
            if len(offsets):
                if ( len(attr) == 3) and ((attr[2] == WANT_TOKENS) or (attr[2] > 0)):                    
                    toks=v.split(" ")
                    for tok in toks:
                        output += tok
                        if len(offsets) > 1:
                            offset=offsets.pop(0)
                            output += self.zone.contents[offset[1]:offsets[0][0]]
                        else:
                            output += "\t"
                else:
                    output += v
                    if len(offsets) > 1:
                        offset=offsets.pop(0)
                        output += self.zone.contents[offset[1]:offsets[0][0]]
                    elif wasOffsets == False:
                        output += "\t"
            else:
                output += v + "\t"
        return output
    
    def keys(self):
        if self.has_data():
            yield 'name'
            yield 'clas'
            yield 'ttl'
            for attr in self.attrs:
                yield attr[0]

    def value_label(self,key):
        if self.desc.has_key(key):
            if type(self.desc[key]) == list:
                return self.desc['']
            return self.desc[key]        
        elif (key == 'name'):
            return _('Domain Name')
        elif (key == 'clas'):
            return None;
        elif (key == 'ttl'):
            return _('Cache Time To Live')
        else:
            return None
        
    def value_labels(self):
        for key in self.keys():
            if key == 'clas':
                continue
            # the per-record 'class' is redundant:
            # it is a named syntax error to specify a different
            # class to the zone class.
            if self.desc.has_key(key):
                val=getattr(self,key)
                t=type(val)
                if ( t == str) or (t == int):                   
                    yield[ self.desc[key], val, 0]
                else:
                    v=''
                    if val.__class__ == DNS.Name:
                        v = str(val)
                        yield[ self.desc[key], v, 0 ]
                    else:
                        if type(self.desc[key]) == list:
                        # handle records with variable length attr lists:
                            for descr in self.desc[key]:
                                attr, desc = descr
                                if hasattr(val,attr) :
                                    a=getattr(val,attr)
                                    v=None
                                    if callable(a):
                                        v = a()
                                    else:
                                        v = a                                     
                                    if type(v) == types.InstanceType:
                                        if v.__class__ == DNS.Name:
                                            v = str(v)
                                        elif hasattr(v,'label'):
                                            v = v.label()
                                        elif hasattr(v,'__str__'):
                                            v = str(v)
                                        elif hasattr(v,'value'):
                                            v = v.value
                                    if( v != None ):
                                        yield( desc, v, 0 )
                        else:
                            if( hasattr(val,'label')):
                                v = val.label()
                            else:
                                v = str(val)
                            yield[ self.desc[key], v, val.__class__ == DNS.Base64 ]
        yield [ _('Cache Time To Live'), self.TTL().label(), 0]
         
    def max_value_label(self):
        m=0
        l=0
        for key in self.keys():
            if self.desc.has_key(key):
                l = len(self.desc[key])
                if l > m:
                    m = l
        return m
        
    def is_directive(self):
        return hasattr(self,'directive')

    def is_mapping(self):
        return hasattr(self,'mapping')
        
    def Name(self):
        n=self.name
        i=self.index
        while((n == None) and (i > 0)):
            i-=1
            if (self.file==None) and (self.zone.records[i].file != None):
                continue
            n = self.zone.records[i].name
        if (n == None):
            zone=self.zone
            if zone.type == DNS.FORWARD :
                n=DNS.Name(zone.zone_origin, zone)
            elif zone.type == DNS.IPV4.REVERSE :
                n=DNS.IPV4.ReverseAddress(zone.zone_origin, zone)
            elif zone.type == DNS.IPV6.REVERSE_ARPA:
                n=DNS.IPV6.ReverseARPAddress(zone.zone_origin, zone)
            elif zone.type == DNS.IPV6.REVERSE_INT:
                n=DNS.IPV6.ReverseINTAddress(zone.zone_origin, zone)
            elif zone.type == DNS.NSAP.REVERSE:
                n=DNS.NSAP.ReverseAddress(zone.zone_origin, zone)
        return n

    def Class(self):
        c=self.clas
        n=self.name
        i=self.index
        while((c == None) and (n==None) and (i > 0)):
            i-=1
            if (self.file==None) and (self.zone.records[i].file != None):
                continue            
            n = self.zone.records[i].name
            c = self.zone.records[i].clas
        if( c == None ):
            if( self.zone.clas == None ):
                return 'IN'
            else:
                return self.zone.clas            
        return c
    
    def TTL(self):
        if( self.ttl == None ):
            return self.zone.ttl
        return self.ttl

    def Type(self):
        return str(self.__class__).split('.')[-1]

    def widget(self, parent, gui, key):
        attr = getattr(self,key)
        debug( '   RR WIDGET: ',key,attr.__class__)
        if type(attr) == type(None):            
            if( key == 'name'):
                attr = self.Name()
            elif ( key == 'ttl'):
                attr = self.TTL() 
        clas = type(attr)
        if( clas == types.InstanceType ):
            clas = str(attr.__class__)
            clas = clas.replace('.','_')
            debug( 'CLAS: ',clas)
        elif clas == int:
            clas = 'int_widget'
        elif clas == str:
            clas = 'Widget'
        elif( clas == types.NoneType ):
            debug( 'UNKNOWN NULL ATTRIBUTE:', key)
            clas='Widget'
        else:
            debug( 'CLASS == ',clas)
            clas='Widget'
        if ( hasattr(gui, clas) ):
            debug( '  WIDGET:',attr.__class__,clas)
            return getattr(gui,clas)(parent,self,attr,key)
        else:
            return gui.Widget(parent,self,attr,key)            
                
    def __init__(self,zone,name,clas,ttl,tokens, offsets, type_token = -1):
        self.zone=zone
        self.name=name
        self.clas=clas
        self.ttl=ttl
        self.tokens=tokens
        self.offsets=offsets
        self.rdata_token = type_token
        self.index=-1
        if(tokens != None):
        # Existing Record from Zone:
        #    debug('RR.__init__: LOAD RR:',self.tokens,type_token)
            self.parse()
            if self.has_data():
                self.index=len(zone.records)
                zone.records.append(self)
                if hasattr(zone,'loaded') and hasattr(zone,'additions') and  zone.loaded:
                    zone.additions+=1
                elif hasattr(zone,'additions'):
                    zone.additions=0
        else:
        # New Record from GUI:
            debug('RR.__init__: NEW RR:')
            self.offsets=[]
            self.tokens=[]
            if type(name) == str:
                if(len(name)==0):
                    name=None
                else:
                    name=DNS.Name(name,zone)
            if name==None:
                if zone.type == DNS.FORWARD:
                    self.name = DNS.Name(zone.zone_origin,zone)
                elif zone.type == DNS.IPV4.REVERSE:
                    self.name = DNS.IPV4.ReverseAddress(zone.zone_origin,zone)
                elif zone.type == DNS.IPV6.REVERSE_ARPA:
                    self.name = DNS.IPV6.ReverseARPAddress(zone.zone_origin,zone)
                elif zone.type == DNS.IPV6.REVERSE_INT:
                    self.name = DNS.IPV6.ReverseINTAddress(zone.zone_origin,zone)
                elif zone.type == DNS.NSAP.REVERSE:
                    self.name = DNS.NSAP.ReverseAddress(zone.zone_origin,zone)                
            for attr in self.attrs:
                newAttr=None
                if (len(attr) > 2):
                    if attr[2] == WANT_ZONE:
                        newAttr=attr[1](None, self.zone)
                    elif attr[2] == WANT_TOKENS:
                        newAttr=attr[1]( [] )
                    elif attr[2] == WANT_ZONE_AND_TOKENS:
                        newAttr=attr[1]( self.zone, [] )
                else:
                    try:
                        newAttr=attr[1](None)
                    except TypeError:
                        if attr[1] == int:
                            newAttr = 0
                        else:
                            newAttr=None                        
                setattr(self, attr[0], newAttr)
            self.new_record=True
            debug('NEW', self.Type(), ' RECORD HAS INDEX:', self.index)
            
    def update(s):
        if hasattr(s.zone,'SOA') and hasattr(s.zone.SOA,'serial'):            
            s.zone.SOA.serial += 1
        s.zone.dirty = True
        debug('RR.update:', s.index, hasattr(s,'new_record'))
        if hasattr(s,'new_record') and (s.index == -1):
            if hasattr(s.zone,'additions'):
                s.zone.additions += 1
            s.index=None
            if s.name != None:
                for r in s.zone.records:
                    if r.has_data() and (r.name != None) and (str(r.name)==str(s.name)):
                        s.name = None
                        s.index=r.index+1
                        debug( 'NEW RR: INSERT AT INDEX',s.index)
                        break
            if (s.index != None) and (s.index < len(s.zone.records)):
                while ((s.index < len(s.zone.records)) and (s.zone.records[s.index].name == None)):
                    s.index+=1
                debug( 'NEW RR: INSERTING AT INDEX',s.index                    )
                s.zone.records.insert(s.index,s)
                for i in range(s.index+1,len(s.zone.records)):                    
                    s.zone.records[i].index+=1
            else:            
                s.index = len(s.zone.records)
                s.zone.records.append(s)
                debug( 'NEW RR: APPEND', s.index)
        else:
            debug( 'NEW RR: UPDATE',s.index)
        
    def __getitem__(self, k):
        return getattr(self,k)
    

class Directive(RR):
    directive=True
    def value(self):
        return ''

class A(RR):
    attrs=[ [ 'ADDRESS', DNS.IPV4.Address ] ]
    desc={ '' : _('IPv4 Address'),
           'ADDRESS' : _('IPv4 Address')
         }
    types=[ DNS.FORWARD ]
    mapping=True
#    not_origin=True
    
class AAAA(RR):
    attrs=[ [ 'ADDRESS', DNS.IPV6.AAAA ] ]
    desc={ '' : _('IPv6 Address'),
           'ADDRESS' : _('IPv6 Address')
         }
    types=[ DNS.FORWARD ]
    mapping=True
#    not_origin=True
    
class A6(RR):
    attrs=[ [ 'ADDRESS', DNS.IPV6.A6, WANT_ZONE_AND_TOKENS ] ]
    desc={ '' : _('Chained IPv6 Address'),
           'ADDRESS' :
            [ [ 'prefix', _('Prefix Length') ],
              [ 'address', _('IPv6 Address Suffix') ],              
              [ 'prefix_name', _('Prefix Name') ]
            ]           
         }
    types=[ DNS.FORWARD ]
    mapping=True
#    not_origin=True
    
class AFSDB(RR):
    attrs=[ [ 'SUBTYPE', int ], [ 'HOSTNAME', DNS.Name ] ]
    desc={ '' : _('Andrew File System Database Server'),
           'ADDRESS' : _('AFSDB Server Address')
         }
    types=[ DNS.FORWARD ]
    
class CERT(RR):
    attrs=[ [ 'TYPE', DNS.CertType ], [ 'KEYTAG', DNS.KeyTag ],
            [ 'ALGORITHM', DNS.KeyAlgorithm ], [ 'CRL', DNS.Base64, WANT_TOKENS ]
          ]
    desc={ '' : _('Digital Certificate'),
           'TYPE': _('Certificate Type'),
           'KEYTAG': _('Key Tag Checksum'),
           'ALGORITHM': _('Key Encryption Algorithm')
         }
    types=[ DNS.FORWARD, DNS.IPV4.REVERSE, DNS.IPV6.REVERSE_ARPA, DNS.IPV6.REVERSE_INT, DNS.NSAP.REVERSE ]
    
class CNAME(RR):
    attrs=[ [ 'CNAME', DNS.Name, WANT_ZONE ] ]
    desc={ '': _('Alias'),
           'CNAME': _('Canonical Name')
         }
    mapping=True
    types=[ DNS.FORWARD, DNS.IPV4.REVERSE, DNS.IPV6.REVERSE_ARPA, DNS.IPV6.REVERSE_INT, DNS.NSAP.REVERSE  ]

class DNAME(RR):
    attrs=[ [ 'DNAME', DNS.Name, WANT_ZONE ] ]
    desc={ '': _('Alias'),
           'DNAME': _('Canonical Domain Name')
         }
    mapping=True
    types=[ DNS.FORWARD, DNS.IPV4.REVERSE, DNS.IPV6.REVERSE_ARPA, DNS.IPV6.REVERSE_INT, DNS.NSAP.REVERSE  ]

class DS(RR):
    attrs=[ [ 'TAG', DNS.KeyTag ] , [ 'ALGORITHM', DNS.KeyAlgorithm ], [ 'DIGEST_TYPE' ,  DNS.KeyDigest ],
            [ 'DIGEST', DNS.Base64, WANT_TOKENS ]
          ]
    desc= { '': _('Delegation Signer'),
            'TAG': _('Key Tag Checksum'),
            'ALGORITHM': _('Key Encryption Algorithm'),
            'DIGEST_TYPE': _('Type of Digest'),
            'DIGEST': _('Digest')
          }
    types=[ DNS.FORWARD, DNS.IPV4.REVERSE, DNS.IPV6.REVERSE_ARPA, DNS.IPV6.REVERSE_INT, DNS.NSAP.REVERSE  ]
    
class HINFO(RR):
    attrs=[ [ 'CPU', str ], ['OS', str] ]
    desc={ '': _('Host Information'),
           'CPU': _('Machine Architecture'),
           'OS': _('Operating System')
         }
    mapping=True
    types=[ DNS.FORWARD ]
           
class ISDN(RR):
    attrs=[ [ 'ISDN_ADDRESS', str ], ['SUBADDRESS',str] ]
    desc={ '': _('ISDN Address'),
           'ISDN_ADDRESS': _('ISDN Address'),
           'SUBADDRESS': _('Sub-Address')
         }
    mapping=True
    types=[ DNS.FORWARD ]
        
class KEY(RR):
    attrs=[ [ 'FLAGS', DNS.KeyFlags ], ['PROTOCOL', DNS.KeyProtocol ], ['ALGORITHM', DNS.KeyAlgorithm], ['KEY', DNS.Base64, WANT_TOKENS ] ]
    desc={ '': _('Encryption Key'),
           'FLAGS': _('Flags'),
           'PROTOCOL': _('Protocol'),
           'ALGORITHM': _('Algorithm'),
           'KEY':_('Key')
         }
    types=[ DNS.FORWARD, DNS.IPV4.REVERSE, DNS.IPV6.REVERSE_ARPA, DNS.IPV6.REVERSE_INT, DNS.NSAP.REVERSE ]
    dnssec=True
    
class DNSKEY(KEY):
    pass

class KX(RR):
    attrs=[ [ 'PREFERENCE', int ], ['EXCHANGER', DNS.Name, WANT_ZONE] ]
    desc={ '': _('Key Exchange'),
           'PREFERENCE':_('Priority'),
           'EXCHANGER':_('Server Name')
         }
    types=[ DNS.FORWARD ]
    
class LOC(RR):
    attrs=[ [ 'LOCATION', DNS.Location, WANT_TOKENS ] ]
    desc={ '':_('Geographical Location'),
           'FIXME': _('To Be Done')
         }
    mapping=True
    types=[ DNS.FORWARD ]
    
class MB(RR):
    attrs=[ [ 'MADNAME', DNS.Name, WANT_ZONE ] ]
    desc={ '':_('Mail Box Server (deprecated: use MX)'),
           'MADNAME':_('Mail Box Server Domain Name')
         }
    deprecated=True
    types=[ DNS.FORWARD ]
    
class MD(RR):
    attrs=[ [ 'MADNAME', DNS.Name, WANT_ZONE ] ]
    desc={ '':_('Mail Domain Server (deprecated: use MX)'),
           'MADNAME':_('Mail Box Server Domain Name')
         }
    deprecated=True
    types=[ DNS.FORWARD ]

class MF(RR):
    attrs=[ [ 'MADNAME', DNS.Name, WANT_ZONE ] ]
    desc={ '':_('Mail Forwarder Server (deprecated: use MX)'),
           'MADNAME':_('Mail Box Server Domain Name')
         }
    deprecated=True
    types=[ DNS.FORWARD ]
    
class MG(RR):
    attrs=[ ['MGMNAME', DNS.Name, WANT_ZONE ] ]
    desc={ '':_('Mail Group (deprecated: use MX)'),
           'MGMNAME':_('Mail Group Member')
         }
    deprecated=True
    types=[ DNS.FORWARD ]
    
class MINFO(RR):
    attrs=[ [ 'RMAILBX', DNS.Name, WANT_ZONE ], [ 'EMAILBX', DNS.Name, WANT_ZONE ] ]
    desc={ '':_('Mail List Information (deprecated: use MX)'),
           'RMAILBX':_('Responsible Mail Box'),
           'EMAILBX':_('Error Mail Box')
         }
    deprecated=True
    types=[ DNS.FORWARD ]
    
class MR(RR):
    attrs=[ [ 'NEWNAME', DNS.Name, WANT_ZONE ] ]
    desc={ '':_('Mail Rename Domain Name (deprecated: use MX)'),
           'NEWNAME':_('New Mail Domain Name')
         }
    deprecated=True
    types=[ DNS.FORWARD ]
    
class MX(RR):
    attrs=[ [ 'PREFERENCE', int ], [ 'EXCHANGE', DNS.Name, WANT_ZONE ] ]
    desc={ '': _('Mail Exchange'),
           'PREFERENCE':_('Priority'),
           'EXCHANGE':_('Mail Server Name')
         }
    types=[ DNS.FORWARD ]
    
class NAPTR(RR):
    attr=[ [ 'ORDER', int ], [ 'PREFERENCE', int ], [ 'FLAGS', str ],
           [ 'SERVICE', str ], ['REGEXP', str], ['REPLACEMENT', DNS.Name, WANT_ZONE ]
         ]
    desc={ '': _('Naming Authority Pointer'),
           'ORDER': _('Order'),
           'PREFERENCE': _('Priority'),
           'FLAGS':_('Flags'),
           'SERVICE':_('Service'),
           'REPLACEMENT':_('Replacement')
         }
    types=[ DNS.FORWARD ]

class NS(RR):
    attrs=[ [ 'NSDNAME', DNS.Name, WANT_ZONE ] ]
    desc={ '': _('Name Server'),
           'NSDNAME':_('Server Domain Name')
         }
    types=[ DNS.FORWARD, DNS.IPV4.REVERSE, DNS.IPV6.REVERSE_ARPA, DNS.IPV6.REVERSE_INT, DNS.NSAP.REVERSE ]
    
class NSAP(RR):
    attrs=[ [ 'ADDRESS', DNS.NSAP.Address ] ]
    desc={'': 'OSI Network Service Access Point Address',
          'ADDRESS': _('OSI Network Service Access Point Address')
         }
    types=[ DNS.FORWARD ]
#    not_origin=True
    
class NXT(RR):
    attrs=[ [ 'NXTNAME', DNS.Name, WANT_ZONE ], [ 'RRTYPES', DNS.List, WANT_TOKENS ] ]
    desc={'': _('Next Resource'),
          'NXTNAME': _('Name of Next Resource'),
          'RRTYPES': _('List of Resource Record Types')
         }
    types=[ DNS.FORWARD, DNS.IPV4.REVERSE, DNS.IPV6.REVERSE_ARPA, DNS.IPV6.REVERSE_INT, DNS.NSAP.REVERSE ]
    dnssec=True

class NSEC(NXT):
    pass

class PTR(RR):
    attrs=[ [ 'PTRDNAME', DNS.Name, WANT_ZONE ] ]
    desc={ '': _('Reverse Address Mapping'),
           'PTRDNAME':_('Host Name')
         }
    mapping=True
    types=[ DNS.IPV4.REVERSE, DNS.IPV6.REVERSE_ARPA, DNS.IPV6.REVERSE_INT, DNS.NSAP.REVERSE  ]
    
class PX(RR):
    attrs=[ [ 'MAP822', DNS.Name ], [ 'MAPX400',  DNS.Name, WANT_ZONE ] ]
    desc={ '': _('Pointer to RFC822/X.400 Mapping'),
           'MAP822': _('RFC822 Domain Name'),
           'MAPX400':_('X.400 Domain Name')
         }
    types=[ DNS.FORWARD ]
    
class RP(RR):
    attrs=[ [ 'MAILBOX', DNS.Email ], [ 'TXTDNAME', DNS.Name, WANT_ZONE ] ]
    desc={ '': _('Responsible Person'),
           'MAILBOX':_('Email Address'),
           'TXTDNAME':_('Domain Name of TXT Record')
         }
    deprecated=True
    types=[ DNS.FORWARD ]
    
class RT(RR):
    attrs=[ [ 'PREFERENCE', int ], [ 'INTERMEDIATE', DNS.Name, WANT_ZONE ] ]
    desc={ '': _('Route Through'),
           'PREFERENCE':_('Preference'),
           'INTERMEDIATE':_('Gateway Address Domain Name')
         }
    deprecated=True
    types=[ DNS.FORWARD ]
        
class SIG(RR):
    attrs=[ [ 'TYPE', DNS.RRType  ], [ 'ALGORITHM', DNS.KeyAlgorithm ], ['NLABELS', int],
            [ 'TTLV', DNS.TTL ], [ 'EXPIRE', DNS.Timestamp ], ['INCEPT', DNS.Timestamp ],
            [ 'KEYTAG', DNS.KeyTag ],  [ 'SIGNER', DNS.Name, WANT_ZONE ],
            ['SIGNATURE', DNS.Base64, WANT_TOKENS]
          ]
    desc={ '': _('Digital Signature Validation'),
           'TYPE': _('Type'),
           'ALGORITHM': _('Algorithm'),
           'NLABELS': _('Number of Labels'),           
           'TTLV': _('Time To Live'),
           'EXPIRE': _('Expire'),
           'INCEPT':_('Incept'),
           'KEYTAG':_('Key Tag'),
           'SIGNER':_('Signer DNS Name'),
           'SIGNATURE':_('Digital Signature')
         }
    types=[ DNS.FORWARD ]
    dnssec=True

class RRSIG(SIG):
    pass

class SOA(RR):
    attrs=[ [ 'MNAME',  DNS.Name, WANT_ZONE  ],
            [ 'RNAME', DNS.Email, WANT_ZONE  ],
            [ 'SERIAL',  int ],
            [ 'REFRESH', DNS.TTL ],
            [ 'RETRY',   DNS.TTL ],
            [ 'EXPIRE',  DNS.TTL ],
            [ 'MINIMUM', DNS.TTL ]
          ]
    desc={'': _('Zone Authority Information'),
          'MNAME': _('Authoritative Name Server'),
          'RNAME': _('Responsible Person E-Mail Address'),
          'SERIAL':_('Zone Modification Serial Number'),
          'REFRESH':_('Refresh Interval'),
          'RETRY':_('Refresh Retry Interval'),
          'EXPIRE':_('Expiration Interval'),
          'MINIMUM':_('Default Minimum Cache TTL')
         }
    types=[ DNS.FORWARD, DNS.IPV4.REVERSE, DNS.IPV6.REVERSE_ARPA, DNS.IPV6.REVERSE_INT ]
    unique=True
    def __init__(self,zone,name,clas,ttl,tokens,offsets,type_token = -1):
        RR.__init__(self,zone,name,clas,ttl,tokens,offsets,type_token)
        self.zone.SOA=self
        if( self.zone.ttl == None ):
            self.zone.ttl = self.MINIMUM

    def out(s, origin=None):
        if( len(s.offsets) > 0):
            return RR.out(s,origin)
        # implement "traditional" multi-line SOA formatting for new SOA records:
        out = ''
        if( s.name != None):
            out += s.name.relative_value(origin) + "\t"
        if( s.clas != None):
            out += str(s.clas) + "\t"
        if( s.ttl != None):
            out += s.ttl.value() + "\t"
        out += "SOA\t"
        out += s.MNAME.value() + "\t"
        out += s.RNAME.value() + " (\t" + str(s.SERIAL)  + "\n"
        out += "\t\t\t\t\t\t" + s.REFRESH.value() + "\n"
        out += "\t\t\t\t\t\t" + s.RETRY.value() + "\n"
        out += "\t\t\t\t\t\t" + s.EXPIRE.value() + "\n"
        out += "\t\t\t\t\t\t" + s.MINIMUM.value() + " )"
        return out

class SRV(RR):
    attrs=[ ['PRIORITY', int], [ 'WEIGHT', int ], [ 'PORT', int ], ['TARGET', DNS.Name, WANT_ZONE ] ]
    desc={'':_('Service Information'),
          'PRIORITY':_('Priority'),
          'WEIGHT':_('Weight'),
          'PORT':_('Service Port Number'),
          'TARGET':_('Server Name')
         }
    types=[ DNS.FORWARD ]

class SSHFP(RR):
   attrs=[ ['ALTYPE', int], ['FPTYPE', int], ['FP', str] ]
   desc={'':_('SSH Fingerprint'),
               'ALTYPE':_('Algorithm type'),
               'FPTYPE':_('Fingerprint type'),
               'FP':_('Fingerprint')
              }
   types=[ DNS.FORWARD ]

class TXT(RR):
    attrs=[ ['TXT', str] ]
    desc={'': _('Associated Text'),
          'TXT':_('Associated Text String')
         }
    types=[ DNS.FORWARD ]

class NULL(TXT):
    pass

class WKS(RR):
    attrs=[ ['ADDRESS', DNS.IPV4.Address ],
            ['PROTOCOL', int ],
            ['SVCS', DNS.List ]
          ]
    desc={ '':_('Well Known Service (deprecated: use SRV)'),
           'ADDRESS':_('IPv4 Address'),
           'PROTOCOL':_('Service IP Protocol Number'),
           'SVCS':_('Bit Map Specifying Port Numbers')
         }
    deprecated=True
    types=[ DNS.FORWARD ]
    
class X25(RR):
    attrs=[ [ 'PSDNADDRESS', str ] ]
    desc={'':'X25 Address',
          'PSDNADDRESS':_('PSDN Address')
         }
    mapping=True
    types=[ DNS.FORWARD ]
    
class ORIGIN(Directive):
    def has_data(self):
        return (len(self.tokens) >= 2)
    
    def parse(self):
        if self.has_data():
            self.origin = self.tokens[1]
#    def __str__(self):
#        return self.origin
    def __init__(self,zone,name,clas,ttl,tokens,offsets,type_token = -1):
        RR.__init__(self,zone,name,clas,ttl,tokens,offsets,type_token)
        zone.origin=self.origin
    
    def value(self):
        return self.origin


class TTL(Directive):
    def has_data(self):
        return (len(self.tokens) >= 2)
    
    def parse(self):
        if len(self.tokens):
            self.ttl = DNS.TTL(self.tokens[1])
#    def __str__(self):
#        return str(self.ttl)
    def __init__(self,zone,name,clas,ttl,tokens,offsets,type_token = -1):
        RR.__init__(self,zone,name,clas,ttl,tokens,offsets,type_token)
        self.zone.ttl=self.ttl

    def value(self):
        return self.ttl.value()

class INCLUDE(Directive):
    filename=''
    origin=None
    previous_origin=None

    def has_data(self):
        return (len(self.tokens) >= 2)
    
    def parse(self):
        if self.has_data():
            self.filename = self.tokens[1]
            if( len(self.tokens)>2):
                self.origin = self.tokens[2]
                self.previous_origin=self.zone.origin
                self.zone.origin = self.origin
                
    def value(self):
        v = self.filename
        if (self.origin != None):
            v+= " " + self.origin
        return v

    def __str__(self):
        return self.value()

    
class GENERATE(Directive):
    range=[]
    
    def has_data(self):
        return (len(self.tokens) >= 2)
    
    def parse(self):
        if self.has_data():
            self.range=self.tokens[1].split('-')
            if( len(self.range) == 2 ):
                self.range[0]=int(self.range[0])
                self.range[1]=int(self.range[1])
                
    def generate(self):
        pass

    def range(self,vmin,vmax):
        range[0]=vmin
        range[1]=vmax

    def value(self):
        val=str(self.range[0]) + '-' + str(self.range[1])
        offsets=self.offsets[1:]
        tokens=self.tokens[2:]
        for token in tokens:
            if(len(offsets)>1):
                offset=offsets.pop(0)
                val+=self.zone.contents[offset[1]:offsets[0][0]]
            else:
                val+"\t"
            val+=token
        return val

RDATA={ 'A': A, 'A6':A6, 'AAAA':AAAA, 'AFSDB':AFSDB, 'CERT':CERT, 'CNAME':CNAME, 'DNAME':DNAME,
        'DNSKEY':DNSKEY, 'DS': DS,
        'HINFO':HINFO, 'ISDN': ISDN, 'KEY':KEY, 'KX': KX, 'LOC': LOC, 'MB': MB, 'MD':MD,'MF':MF,'MG':MG,'MINFO':MINFO,
        'MR':MR, 'MX':MX, 'NAPTR':NAPTR, 'NS':NS, 'NSAP':NSAP, 'NSEC':NSEC, 'NXT':NXT, 'NULL':NULL, 'PTR': PTR,
        'PX':PX, 'RP':RP, 'RRSIG':RRSIG, 'SIG':SIG, 'SOA': SOA, 'TXT':TXT, 'SRV':SRV, 'SSHFP':SSHFP, 'WKS':WKS, 'X25':X25,
        # Directives:
        'INCLUDE': INCLUDE, 'GENERATE': GENERATE, 'ORIGIN': ORIGIN, 'TTL': TTL       
      }

DNSSEC_RRs=[ 'DNSKEY', 'KEY', 'NSEC', 'NXT', 'RRSIG', 'SIG' ]; 

def parse(zone, continuation, tokens, offsets ):
    tok=0
    if ( tokens[0][0] == '$'):
    # $ORIGIN, $TTL, $INCLUDE, $GENERATE:
        directive=tokens[0][1:].upper()
        if RDATA.has_key(directive):
            return RDATA[directive](zone, None,None,None, tokens, offsets)
    # RR:
    name=None
    if not continuation:
        if zone.type == DNS.FORWARD :
            name=DNS.Name(tokens[0], zone)
        elif zone.type == DNS.IPV4.REVERSE :
            name=DNS.IPV4.ReverseAddress(tokens[0],zone)
        elif zone.type == DNS.IPV6.REVERSE_ARPA:
            name=DNS.IPV6.ReverseARPAddress(tokens[0],zone)
        elif zone.type == DNS.IPV6.REVERSE_INT:
            name=DNS.IPV6.ReverseINTAddress(tokens[0],zone)
        elif zone.type == DNS.NSAP.REVERSE:
            name=DNS.NSAP.ReverseAddress(tokens[0],zone)            
        if (name == None) or (str(name)==''):
            badRR=NULL(zone, None,None,None,None, offsets)
            return zone.OUT
        tok=1
    if name != None:
        if not zone.In(str(name)):
            badRR=NULL(zone, None,None,None,None, offsets)
            return zone.OUT
    token=None
    clas=None
    ttl=None
    next_tok=tok
    for token in tokens[tok:min(tok+3,len(tokens))]:
        if token.upper() in RDATA.keys():
            break
        elif (clas==None) and (token.upper() in RR.classes):
            clas=token.upper()
            next_tok+=1
        elif(ttl==None):
            ttl=DNS.TTL(token)
            if( ttl.seconds == None ):
                ttl=None
                token=None
                break
            next_tok+=1
        token=None
    if token==None and (next_tok < len(tokens)):
        token=tokens[next_tok]
    if (token != None) and RDATA.has_key(token.upper()):
        return RDATA[token.upper()](zone, name, clas, ttl, tokens, offsets, next_tok+1)
    else:
        badRR=NULL(zone, None,None,None,None, offsets)
        return None


